//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Reflection;
using System.IO;
using System.Globalization;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.Biz
{
    public class DcAppSettingNotFound : ApplicationException
    {
        public string SettingName;
        public DcAppSettingNotFound(string msg, string settingName)
            : base(msg)
        {
            SettingName = settingName;
        }
    }
    
    public class BizGlobal
    {
        protected virtual MySqlDLGlobal GetMySqlDLGlobal()
        {
            return new MySqlDLGlobal();
        }

        protected virtual AssemblyWrapper LoadAssembly(string assemblyFilePath)
        {
            return new AssemblyWrapper(Assembly.LoadFrom(assemblyFilePath));
        }

        protected virtual StringResHelper GetStringResHelper(
            string baseName, 
            AssemblyWrapper assembly
            )
        {
            return new StringResHelper(baseName, assembly);
        }

        protected virtual UserInfo GetUserInfo(string userLogin, DataConnection con)
        {
            return new UserInfo(userLogin, con);
        }

        protected virtual LogHelper GetLogHelper(string logFile)
        {
            return new LogHelper(logFile);
        }

        protected virtual DataConnection GetDataConnection(string connString)
        {
            return new DataConnection(connString);
        }

        protected virtual ResourceHelper GetResourceHelper(AssemblyWrapper assembly)
        {
            return new ResourceHelper(assembly);
        }

        protected virtual Notificator GetNotificator(UserInfo serviceUserInfo, SettingKeeper sk, DataConnection con)
        {
            return new Notificator(serviceUserInfo, sk, con);
        }

        protected virtual DBConstAccess GetDBConstAccess(DBConnection con)
        {
            return new DBConstAccess(con);
        }

        protected string ReplaceServerInUrl(string url, string server)
        {
            string urlStart = "http://";
            StringBuilder retVal = new StringBuilder(urlStart);
            retVal.Append(server);
            retVal.Append(url.Substring(url.IndexOf("/", urlStart.Length)));
            return retVal.ToString();
        }

        protected virtual DBFieldDefKeeper GetDBFieldDefKeeper(string applicationFolder)
        {
            return new DBFieldDefKeeper(applicationFolder);
        }

        public virtual bool GetIsSetupNeeded()
        {
            string[] requiredKeys = new string[]
            {
                SettingFileKeys.LogFile,
                SettingFileKeys.ConnectionString,
                SettingFileKeys.ServiceConnectionString,
                SettingFileKeys.AttachmentFolder,
                SettingFileKeys.ServiceUserLogin
            };

            bool setupIsNeeded = false;
            foreach (string key in requiredKeys)
            {
                if (String.IsNullOrEmpty(DCAppSettings.Current[key]))
                {
                    setupIsNeeded = true;
                    break;
                }
            }

            return setupIsNeeded;
        }

        public virtual void InitializeResources(string mappedRequestApplicationPath, SettingKeeper sk)
        {
            //resource folder

            string resourceFileFolder = null;

            if (DCAppSettings.Current[SettingFileKeys.ResourceFileFolder] != null &&
                DCAppSettings.Current[SettingFileKeys.ResourceFileFolder] != ""
                )
            {
                resourceFileFolder = DCAppSettings.Current[SettingFileKeys.ResourceFileFolder];
            }
            else
            {
                resourceFileFolder = Path.Combine(mappedRequestApplicationPath, "bin");
            }

            sk.ResourceFileFolder = resourceFileFolder;

            //application culture
            string applicationCulture = CultureInfo.CurrentUICulture.Name;

            //GlobalStringConstants.NeutralCulture;

            //if (DCAppSettings.Current[SettingFileKeys.ApplicationCulture] != null &&
            //    DCAppSettings.Current[SettingFileKeys.ApplicationCulture] != ""
            //    )
            //{
            //    applicationCulture = DCAppSettings.Current[SettingFileKeys.ApplicationCulture];
            //}

            sk.ApplicationCulture = applicationCulture;

            //init resources
            AssemblyWrapper assembly = null;
            if (applicationCulture != GlobalStringConstants.NeutralCulture)
            {
                string resSubFolder = Path.Combine(resourceFileFolder, applicationCulture);
                string assemblyFilePath = Path.Combine(
                    resSubFolder, GlobalStringConstants.ResourceAssemblyFileName
                    );

                if (FileWrapper.Current.Exists(assemblyFilePath))
                {
                    assembly = LoadAssembly(assemblyFilePath);
                }
            }

            if (assembly == null)
            {
                string assemblyFilePath = Path.Combine(
                    resourceFileFolder, GlobalStringConstants.ResourceAssemblyFileName
                    );
                if (FileWrapper.Current.Exists(assemblyFilePath))
                {
                    assembly = LoadAssembly(assemblyFilePath);
                }
                else
                {
                    throw new ApplicationException(
                        String.Format("Can't open any resource file. Check existence of {0}", assemblyFilePath)
                        );
                }
            }

            ResourceHelper.Current = GetResourceHelper(assembly);

            ResourceHelper.StrForLog = GetStringResHelper(
                GlobalStringConstants.StringsForLogResourcesId,
                assembly
                );

            ResourceHelper.StrForUI = GetStringResHelper(
                GlobalStringConstants.StringsForUIResourcesId,
                assembly
                );

            ResourceHelper.StrForEx = GetStringResHelper(
                GlobalStringConstants.StringsForExResourcesId,
                assembly
                );

        }

        public virtual Dictionary<string, object> Init(
            string machineName, 
            string requestApplicationPath,
            string mappedRequestApplicationPath,
            string absoluteUrl,
            int port
            )
        {

            //init variable
            SettingKeeper sk = new SettingKeeper();

            InitializeResources(mappedRequestApplicationPath, sk);
            
            //init log file
            if (DCAppSettings.Current[SettingFileKeys.LogFile] == null)
            {
                throw new DcAppSettingNotFound(
                    String.Format(
                        ResourceHelper.StrForEx[ExResKeys.ConfigurationFileKeyIsNotFound],
                        SettingFileKeys.LogFile
                        ),
                    SettingFileKeys.LogFile
                    );
            }
            string logFile = DCAppSettings.Current[SettingFileKeys.LogFile];
            LogHelper.Current = GetLogHelper(logFile);
            LogHelper.Current.MakeDelimiter();
            LogHelper.Current.AddStr(ResourceHelper.StrForLog[LogResKeys.ApplicationStart]);
            sk.ServerName = machineName;

            string domain = DCAppSettings.Current[SettingFileKeys.Domain];
            if (domain == null)
            {
                throw new DcAppSettingNotFound(
                    String.Format(
                        ResourceHelper.StrForEx[ExResKeys.ConfigurationFileKeyIsNotFound],
                        SettingFileKeys.Domain
                        ),
                    SettingFileKeys.Domain
                    );
            }
            sk.Domain = domain;
            //string[] domainParts = domain.Split(new char[] { '.' });

            //StringBuilder sb = new StringBuilder();
            //sb.AppendFormat("LDAP://{0}/", machineName);
            //for (int i = 0; i < domainParts.Length; i++)
            //{
            //    sb.AppendFormat("DC={0}", domainParts[i]);
            //    if (i != domainParts.Length - 1)
            //    {
            //        sb.Append(",");
            //    }
            //}
            //string adAccessPath = sb.ToString();

            //sk.ADAccessPath = adAccessPath;
            //sk.MessageTemplate = Path.Combine(mappedRequestApplicationPath, "msgtempl.txt");

            sk.ApplicationFolder = mappedRequestApplicationPath;

            if (DCAppSettings.Current[SettingFileKeys.AddDomainInUrls] != null &&
            DCAppSettings.Current[SettingFileKeys.AddDomainInUrls] != ""
            )
            {
                sk.AddDomainInUrls =
                    bool.Parse(DCAppSettings.Current[SettingFileKeys.AddDomainInUrls]);
            }
            else
            {
                sk.AddDomainInUrls = DefaultSettingValues.AddDomainInUrls;
            }

            if (sk.AddDomainInUrls)
            {
                sk.ApplicationUrl = String.Format(
                    "http://{0}.{1}{2}",
                    machineName,
                    domain,
                    requestApplicationPath
                    );
            }
            else
            {
                string portPart = 
                    port != 80 ? ":" + port.ToString() : String.Empty;

                sk.ApplicationUrl = String.Format(
                    "http://{0}{1}{2}",
                    machineName,
                    portPart,
                    requestApplicationPath
                    );
            }


            //Init service user
            string connString = DCAppSettings.Current[SettingFileKeys.ConnectionString];
            if (connString == null)
            {
                throw new ApplicationException(ResourceHelper.StrForEx[ExResKeys.ConnectionStringIsNotSet]);
            }
            sk.ConnectionString = connString;

            if (DCAppSettings.Current[SettingFileKeys.ServiceConnectionString] != null &&
                DCAppSettings.Current[SettingFileKeys.ServiceConnectionString] != ""
                )
            {
                sk.ServiceConnectionString = DCAppSettings.Current[SettingFileKeys.ServiceConnectionString];
            }
            else
            {
                sk.ServiceConnectionString = connString;
            }


            if (DCAppSettings.Current[SettingFileKeys.AttachmentBufferSize] != null &&
                DCAppSettings.Current[SettingFileKeys.AttachmentBufferSize] != ""
                )
            {
                sk.AttachmentBufferSize = int.Parse(
                    DCAppSettings.Current[SettingFileKeys.AttachmentBufferSize]
                    );
            }
            else
            {
                sk.AttachmentBufferSize = DefaultSettingValues.AttachmentBufferSize;
            }

            if (DCAppSettings.Current[SettingFileKeys.MaxAttachmentSize] != null &&
                DCAppSettings.Current[SettingFileKeys.MaxAttachmentSize] != ""
                )
            {
                sk.MaxAttachmentSize = int.Parse(
                    DCAppSettings.Current[SettingFileKeys.MaxAttachmentSize]
                    );
            }
            else
            {
                sk.MaxAttachmentSize = DefaultSettingValues.MaxAttachmentSize;
            }

            if (DCAppSettings.Current[SettingFileKeys.MaxFilesInFolder] != null &&
              DCAppSettings.Current[SettingFileKeys.MaxFilesInFolder] != ""
              )
            {
                sk.MaxFilesInFolder = int.Parse(
                    DCAppSettings.Current[SettingFileKeys.MaxFilesInFolder]
                    );
            }
            else
            {
                sk.MaxFilesInFolder = DefaultSettingValues.MaxFilesInFolder;
            }

            if (DCAppSettings.Current[SettingFileKeys.AttachmentFolder] != null &&
              DCAppSettings.Current[SettingFileKeys.AttachmentFolder] != ""
              )
            {
                sk.AttachmentFolder = DCAppSettings.Current[SettingFileKeys.AttachmentFolder];
            }
            else
            {
                sk.AttachmentFolder = Path.Combine(
                    sk.ApplicationFolder,
                    DefaultSettingValues.AttachmentSubFolder
                    );
            }

            if (DCAppSettings.Current[SettingFileKeys.EmailMessagingEnabled] != null &&
              DCAppSettings.Current[SettingFileKeys.EmailMessagingEnabled] != ""
              )
            {
                sk.EmailMessagingEnabled = (DCAppSettings.Current[SettingFileKeys.EmailMessagingEnabled] == "true");
            }
            else
            {
                sk.EmailMessagingEnabled = DefaultSettingValues.EmailMessagingEnabled;
            }

            if (DCAppSettings.Current[SettingFileKeys.EmailMessagingServer] != null &&
              DCAppSettings.Current[SettingFileKeys.EmailMessagingServer] != ""
              )
            {
                sk.EmailMessagingServer = DCAppSettings.Current[SettingFileKeys.EmailMessagingServer];
            }



            if (DCAppSettings.Current[SettingFileKeys.EmailMessagingAccount] != null &&
              DCAppSettings.Current[SettingFileKeys.EmailMessagingAccount] != ""
              )
            {
                sk.EmailMessagingAccount = DCAppSettings.Current[SettingFileKeys.EmailMessagingAccount];
            }
            else
            {
                sk.EmailMessagingAccount = "";
            }

            if (DCAppSettings.Current[SettingFileKeys.EmailMessagingPassword] != null &&
              DCAppSettings.Current[SettingFileKeys.EmailMessagingPassword] != ""
              )
            {
                sk.EmailMessagingPassword = DCAppSettings.Current[SettingFileKeys.EmailMessagingPassword];
            }
            else
            {
                sk.EmailMessagingPassword = "";
            }

            if (DCAppSettings.Current[SettingFileKeys.EmailMessagingPort] != null &&
              DCAppSettings.Current[SettingFileKeys.EmailMessagingPort] != ""
              )
            {
                sk.EmailMessagingPort = int.Parse(DCAppSettings.Current[SettingFileKeys.EmailMessagingPort]);
            }
            else
            {
                sk.EmailMessagingPort = DefaultSettingValues.EmailMessagingPort;
            }

            if (DCAppSettings.Current[SettingFileKeys.EmailMessagingSSLEnabled] != null &&
              DCAppSettings.Current[SettingFileKeys.EmailMessagingSSLEnabled] != ""
              )
            {
                sk.EmailMessagingSSLEnabled = (DCAppSettings.Current[SettingFileKeys.EmailMessagingSSLEnabled] == "true");
            }
            else
            {
                sk.EmailMessagingSSLEnabled = DefaultSettingValues.EmailMessagingSSLEnabled;
            }

            if (DCAppSettings.Current[SettingFileKeys.IMWebService] != null &&
              DCAppSettings.Current[SettingFileKeys.IMWebService] != ""
              )
            {
                sk.IMWebService = DCAppSettings.Current[SettingFileKeys.IMWebService];
            }

            if (DCAppSettings.Current[SettingFileKeys.IMWebServiceEnabled] != null &&
              DCAppSettings.Current[SettingFileKeys.IMWebServiceEnabled] != ""
              )
            {
                sk.IMWebServiceEnabled =
                    bool.Parse(DCAppSettings.Current[SettingFileKeys.IMWebServiceEnabled]);
            }
            else
            {
                sk.IMWebServiceEnabled = false;
            }

            if (!String.IsNullOrEmpty(DCAppSettings.Current[SettingFileKeys.AttachmentSavingDialogEnabled]))
            {
                sk.AttachmentSavingDialogEnabled =
                    bool.Parse(DCAppSettings.Current[SettingFileKeys.AttachmentSavingDialogEnabled]);
            }
            else
            {
                sk.AttachmentSavingDialogEnabled = DefaultSettingValues.AttachmentSavingDialogEnabled;
            }

                

            Dictionary<string, object> retVal = new Dictionary<string, object>();

            retVal.Add(AppStateKeys.SettingKeeper, sk);

            //init database
            string dbUpdateResult = "";
            try
            {
                dbUpdateResult = GetMySqlDLGlobal().UpdateDatabase(sk);
            }
            catch (Exception ex)
            {
                LogHelper.Current.WriteException(ex);
            }

            LogHelper.Current.AddStr(
                String.Format(
                    ResourceHelper.StrForLog[LogResKeys.UpdateExecutedWithText],
                    dbUpdateResult
                    )
                );


            DataConnection con = null;
            try
            {
                con = GetDataConnection(sk.ConnectionString);
                con.Open();
            }
            catch (Exception ex)
            {
                LogHelper.Current.WriteException(ex);
                throw new ApplicationException(
                    ResourceHelper.StrForEx[ExResKeys.CannotConnectToDatabase]
                    );
            }
            
            
            UserInfo ui = GetUserInfo(DCAppSettings.Current[SettingFileKeys.ServiceUserLogin], con);
            

            if (ui.Emails.Length == 0)
            {
                throw new ApplicationException(
                    ResourceHelper.StrForEx[ExResKeys.ServiceUserDoNotHaveEmail]
                    );
            }

            retVal.Add(AppStateKeys.ServiceUserInfo, ui);
            LogHelper.Current.AddStr(
                String.Format(ResourceHelper.StrForLog[LogResKeys.ServiceUserInfo], ui.Login, ui.Emails[0])
                );

            retVal.Add(
                AppStateKeys.Notificator, 
                GetNotificator(ui, sk, con));
            retVal.Add(
                AppStateKeys.ApplicationDataConnection,
                con
                );


            DBConstAccess dca = GetDBConstAccess(con.DBConnection);
            FolderTypeDirectory ftd = dca.GetFolderTypeDirectory();
            DBConstKeeper.Current = new DBConstKeeper(
                ftd,
                dca.GetRootFolderDirectory(ftd),
                dca.GetMainRoleDirectory(),
                dca.GetRouteItemTypeDirectory(),
                dca.GetDocStatusDirectory(),
                dca.GetNotificationTypeDirectory(),
                dca.GetEventTypeDirectory(),
                dca.GetCommonDepartment()
                );

            DBFieldDefKeeper.Current = GetDBFieldDefKeeper(sk.ApplicationFolder);

            LogHelper.Current.AddStr(ResourceHelper.StrForLog[LogResKeys.ApplicationInitialized]);
            return retVal;
        }
    }
}
