//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.MySqlDL;


namespace Evpo.DocCirc.Biz
{
    public class DocCardMovementController
    {
        protected UserInfo _userInfo;
        protected DataConnection _con;

        private List<NotificationItem> _notifications;

        public List<NotificationItem> Notifications
        {
            get { return _notifications; }
            set { _notifications = value; }
        }

        private List<EventItem> _events;

        public List<EventItem> Events
        {
            get { return _events; }
            set { _events = value; }
        }

        public DocCardMovementController(UserInfo userInfo, DataConnection con)
        {
            _userInfo = userInfo;
            _con = con;

            _events = new List<EventItem>();
            _notifications = new List<NotificationItem>();
        }

        public void Pass(DocCardItem item, int activeUser, string userNote)
        {
            if (item.DocStatus != DBConstKeeper.Current.DocStatus.UnderCons)
            {
                throw new ApplicationException("Unexpected DocStatus!");
            }

            int? deputy = null;
            if(activeUser != _userInfo.UserId)
            {
                deputy = _userInfo.UserId;
            }

            DocCardRouteItem routeItem = GetRouteItemByUser(item.RoutePoints[item.RoutePointNum - 1], activeUser);
            
            
            if (routeItem == null)
            {
                throw new ApplicationException("RouteItem with ActiveUser is not found!");
            }

            if (!routeItem.UnderCons)
            {
                throw new ApplicationException("RouteItem is not UnderCons!");
            }



            int eventType = DBConstKeeper.Current.EventType.PassDecision;
            if(deputy != null)
            {
                eventType = DBConstKeeper.Current.EventType.PassDecisionByDeputy;
            }
            DateTime now = DateTime.Now;
            _events.Add(
                new EventItem(
                item.Id,
                eventType,
                activeUser,
                deputy,
                item.RoutePointNum,
                now,
                userNote,
                ""
                )
                );
            item.LastMovement = now;
            routeItem.UnderCons = false;
            routeItem.Passed = true;

            AfterRoutePointProcessing(item, activeUser, userNote);

        }


        public void Reject(DocCardItem item, int activeUser, int pointOfReturn, string userNote, bool asAdmin)
        {
            bool routeChange = true;
            Reject(item, activeUser, pointOfReturn, userNote, asAdmin, !routeChange);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="item"></param>
        /// <param name="activeUser"></param>
        /// <param name="pointOfReturn">0 if returns to the draft state</param>
        public void Reject(
            DocCardItem item, 
            int activeUser, 
            int pointOfReturn, 
            string userNote, 
            bool asAdmin, 
            bool routeChange
            )
        {
            if (item.DocStatus != DBConstKeeper.Current.DocStatus.UnderCons)
            {
                throw new ApplicationException("Unexpected DocStatus!");
            }
            DocCardRouteItem routeItem = null;

            if (!routeChange)
            {
                routeItem = GetRouteItemByUser(item.RoutePoints[item.RoutePointNum - 1], activeUser);

                if (routeItem == null)
                {
                    throw new ApplicationException("RouteItem with ActiveUser is not found!");
                }

                if (!routeItem.UnderCons)
                {
                    throw new ApplicationException("RouteItem is not UnderCons!");
                }
            }
            DateTime now = DateTime.Now;

            int? deputy = null;
            if(activeUser != _userInfo.UserId)
            {
                deputy = _userInfo.UserId;
            }

            int eventType = DBConstKeeper.Current.EventType.RejectDecision;
            if (routeChange)
            {
                eventType = DBConstKeeper.Current.EventType.ReturnToDraftDueToRouteChange;
            }
            else if(deputy != null && !asAdmin)
            {
                eventType = DBConstKeeper.Current.EventType.RejectDecisionByDeputy;
            }
            else if(asAdmin)
            {
                eventType = DBConstKeeper.Current.EventType.RejectDecisionByAdministrator;
            }

            _events.Add(
                new EventItem(
                    item.Id,
                    eventType,
                    activeUser,
                    deputy,
                    pointOfReturn,
                    now,
                    userNote,
                    ""
                    )
                );
            item.LastMovement = now;

            for (int i = item.RoutePointNum - 1; i > pointOfReturn - 1; i--)
            {
                foreach (DocCardRouteItem ri in item.RoutePoints[i].RouteItems)
                {
                    ri.UnderCons = false;
                    ri.Passed = false;
                }
            }

            if (pointOfReturn != 0)
            {
                foreach (DocCardRouteItem ri in item.RoutePoints[pointOfReturn - 1].RouteItems)
                {
                    ri.UnderCons = false;
                    ri.Passed = false;
                }
                bool afterReturn = true;
                StepToNextRoutePoint(item, activeUser, pointOfReturn, userNote, afterReturn);
            }
            else
            {
                MakeDraft(item, userNote, _userInfo.UserId, routeChange);
            }
        }

        public void StartConsideration(DocCardItem item, int activeUser)
        {

            item.DocStatus = DBConstKeeper.Current.DocStatus.UnderCons;
            DateTime now = DateTime.Now;
            item.RegistrationTime = now;

            _events.Add(
                new EventItem(
                item.Id,
                DBConstKeeper.Current.EventType.ConsiderationStarted,
                activeUser,
                null,
                0,
                now,
                "",
                ""
                )
                );

            int routePointNum = 1;
            bool afterReturn = true;
            StepToNextRoutePoint(item, activeUser, routePointNum, "", !afterReturn );

            item.LastMovement = now;
        }

        public void Postpone(DocCardItem item, int activeUser, string userNote, bool asAdmin)
        {
            int? deputy = null;
            if(activeUser != _userInfo.UserId)
            {
                deputy = _userInfo.UserId;
            }

            int eventType = DBConstKeeper.Current.EventType.PostponeDecision;
            if(deputy != null && !asAdmin)
            {
                eventType = DBConstKeeper.Current.EventType.PostponeDecisionByDeputy;
            }
            else if(asAdmin)
            {
                eventType = DBConstKeeper.Current.EventType.PostponeDecisionByAdministrator;
            }

            DateTime now = DateTime.Now;

            _events.Add(
                new EventItem(
                    item.Id,
                    eventType,
                    activeUser,
                    deputy,
                    item.RoutePointNum,
                    DateTime.Now,
                    userNote,
                    ""
                    )
                );

            item.LastMovement = now;

        }

        public bool CheckReturnDueToRouteChange(
            DocCardItem item, 
            RoutePoint[] newRoutePoints
            )
        {
            bool returnIsNeeded = false;
            
            List<RouteItem> routeBefore = item.RoutePoints[item.RoutePointNum - 1].RouteItems;


            List<RouteItem> routeAfter = null;

            if (newRoutePoints.Length >= item.RoutePointNum)
            {
                routeAfter = newRoutePoints[item.RoutePointNum - 1].RouteItems;
            }
            else
            {
                returnIsNeeded = true;
            }
                
            Dictionary<int, RouteItem> routeAfterDict = new Dictionary<int, RouteItem>();

            if (!returnIsNeeded)
            {
                foreach (RouteItem ri in routeAfter)
                {
                    if (ri.Id != null)
                    {
                        routeAfterDict.Add(ri.Id.Value, ri);
                    }
                    else
                    {
                        if (ri.RouteItemType == DBConstKeeper.Current.RouteItemType.Decision)
                        {
                            returnIsNeeded = true;
                            break;
                        }
                    }
                }
            }

            if (!returnIsNeeded)
            {
                foreach (RouteItem ri in routeBefore)
                {
                    DocCardRouteItem dcri = (DocCardRouteItem)ri;
                    if (
                        routeAfterDict[ri.Id.Value] == null && 
                        dcri.RouteItemType == DBConstKeeper.Current.RouteItemType.Decision &&
                        dcri.UnderCons
                        )
                    {
                        returnIsNeeded = true;
                        break;
                    }
                }
            }

            return returnIsNeeded;
        }

        protected DocCardRouteItem GetRouteItemByUser(RoutePoint rp, int user)
        {
            List<RouteItem> routeItemList = rp.RouteItems;

            DocCardRouteItem routeItem = null;
            foreach (DocCardRouteItem ri in routeItemList)
            {
                if (ri.User == user)
                {
                    routeItem = ri;
                    break;
                }
            }

            return routeItem;
        }

        protected void StepToNextRoutePoint(
            DocCardItem docCardItem, 
            int activeUser, 
            int routePointNum, 
            string userNote,
            bool afterReturn
            )
        {
            docCardItem.RoutePointNum = routePointNum;

            DateTime now = DateTime.Now;

            int? deputy = null;
            if(activeUser != _userInfo.UserId)
            {
                deputy = _userInfo.UserId;
            }

            _events.Add(
                new EventItem(
                docCardItem.Id,
                DBConstKeeper.Current.EventType.PassedToNextRoutePoint,
                activeUser,
                deputy,
                routePointNum,
                now,
                "",
                ""
                )
                );

            List<RouteItem> routeItems = docCardItem.RoutePoints[routePointNum - 1].RouteItems;

            foreach (DocCardRouteItem item in routeItems)
            {
                if (item.RouteItemType == DBConstKeeper.Current.RouteItemType.Notification)
                {
                    int notifType = DBConstKeeper.Current.NotificationType.NextNotification;
                    if (afterReturn)
                    {
                        notifType = DBConstKeeper.Current.NotificationType.NotificationAfterReturn;
                    }

                    _notifications.Add(
                        new NotificationItem(
                        docCardItem.Id,
                        notifType,
                        item.User,
                        now,
                        "",
                        userNote,
                        _userInfo.UserId,
                        docCardItem.RoutePointNum
                        )
                        );

                }
                else if (item.RouteItemType == DBConstKeeper.Current.RouteItemType.Decision)
                {
                    UserProperties curUserProp = UserHelper.Current.GetUserProperties(item.User, _con);
                    if (curUserProp.AutomaticPass)
                    {
                        
                        _events.Add(
                            new EventItem(
                            docCardItem.Id,
                            DBConstKeeper.Current.EventType.PassDecisionAutomatically,
                            item.User,
                            null,
                            routePointNum,
                            now,
                            "",
                            ""
                            )
                            );
                        item.Passed = true;
                    }
                    else 
                    {
                        int eventType = DBConstKeeper.Current.NotificationType.NextDecision;
                        if(afterReturn)
                        {
                            eventType = DBConstKeeper.Current.NotificationType.DecisionAfterReturn;
                        }

                        _notifications.Add(
                           new NotificationItem(
                           docCardItem.Id,
                           eventType,
                           item.User,
                           now,
                           "",
                           userNote,
                           _userInfo.UserId,
                           docCardItem.RoutePointNum
                           )
                           );

                        if (curUserProp.AllowDeputyNotification)
                        {
                            DeputyItem[] deputies = UserHelper.Current.GetUserDeputies(item.User, _con);
                            foreach (DeputyItem di in deputies)
                            {
                                if (di.Notify)
                                {
                                    _notifications.Add(
                                        new NotificationItem(
                                        docCardItem.Id,
                                        DBConstKeeper.Current.NotificationType.DeputyNextDecision,
                                        di.Deputy,
                                        now,
                                        "",
                                        userNote,
                                        _userInfo.UserId,
                                        docCardItem.RoutePointNum
                                        )
                                        );
                                }
                            }
                        }

                        item.UnderCons = true;
                    }
                }
            }

            AfterRoutePointProcessing(docCardItem, activeUser, userNote);
           
        }

        protected void AfterRoutePointProcessing(DocCardItem item, int activeUser, string userNote)
        {
            if (GetRoutePointIsFinalized(item.RoutePoints[item.RoutePointNum - 1]))
            {
                if (item.RoutePoints.Length > item.RoutePointNum)
                {
                    bool afterReturn = true;
                    StepToNextRoutePoint(item, activeUser, item.RoutePointNum + 1, userNote, !afterReturn);
                }
                else
                {
                    FinalizeDocCard(item);
                }
            }
        }

        protected bool GetRoutePointIsFinalized(RoutePoint routePoint)
        {
            bool allArePassed = true;
            foreach (DocCardRouteItem ri in routePoint.RouteItems)
            {
                if (ri.RouteItemType == DBConstKeeper.Current.RouteItemType.Decision && !ri.Passed)
                {
                    allArePassed = false;
                    break;
                }
            }
            return allArePassed;
        }

        protected void MakeDraft(DocCardItem docCardItem, string userNote, int lastActionUser)
        {
            bool routeChange = true;
            MakeDraft(docCardItem, userNote, lastActionUser, !routeChange);
        }

        protected void MakeDraft(DocCardItem docCardItem, string userNote, int lastActionUser, bool routeChange)
        {
            DateTime now = DateTime.Now;
            docCardItem.RoutePointNum = 0;
            docCardItem.DocStatus = DBConstKeeper.Current.DocStatus.Draft;

            int notifType = DBConstKeeper.Current.NotificationType.ReturnToDraftDueToDecision;
            if (routeChange)
            {
                notifType = DBConstKeeper.Current.NotificationType.ReturnToDraftDueToRouteChange;
            }

            _notifications.Add(
                new NotificationItem(
                    docCardItem.Id,
                    notifType,
                    docCardItem.RegisteringUser,
                    now,
                    "",
                    userNote,
                    lastActionUser,
                    docCardItem.RoutePointNum
                )
                );
        }

        protected void FinalizeDocCard(DocCardItem docCardItem)
        {
            DateTime now = DateTime.Now;

            docCardItem.FinalizingTime = now;
            docCardItem.RoutePointNum = 0;
            docCardItem.DocStatus = DBConstKeeper.Current.DocStatus.Finalized;

            _notifications.Add( 
                new NotificationItem(
                docCardItem.Id,
                DBConstKeeper.Current.NotificationType.Finalization,
                docCardItem.RegisteringUser,
                now,
                "",
                "",
                null,
                0
                )
                );

            _events.Add(
                new EventItem(
                docCardItem.Id,
                DBConstKeeper.Current.EventType.DocCardFinalization,
                _userInfo.UserId,
                null,
                0,
                now,
                "",
                ""
                )
                );
        }


    }
}
