//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.Biz
{

    public class DcUnableToDeleteDocTypeDueToDocCardExistence : ApplicationException
    {
        public Dictionary<int, string> DocCards;
        public DcUnableToDeleteDocTypeDueToDocCardExistence(string msg, Dictionary<int, string> docCards)
            :base(msg)
        {
            DocCards = docCards;
        }
    }

    public class DocTypeData
    {
        protected const int _internalNumberLength = 6;
        private object _thisLock = new object();
        protected DocTypeAccess _dtAccess;
        private UserInfo _userInfo;
        private DataConnection _con;
        /// <summary>
        /// for unit testing
        /// </summary>
        public DocTypeData()
        {
        }

        public DocTypeData(UserInfo userInfo, DataConnection con)
        {
            _dtAccess = new DocTypeAccess(userInfo.UserId, con.DBConnection);
            _userInfo = userInfo;
            _con = con;
        }

        public DocTypeItem[] GetDocTypes(int folderId)
        {
            return _dtAccess.GetDocTypes(folderId);
        }

        public DocTypeItem[] GetDocTypes()
        {
            return _dtAccess.GetDocTypes();
        }

        public void DeleteDocType(int id)
        {
            bool loadRoute = false;

            DocTypeItem item = _dtAccess.LoadDocTypeItem(id, !loadRoute);

            _con.BeginTransaction();
            try
            {
                _dtAccess.DeleteDocType(id);
            }
            catch (Exception ex)
            {
                _con.RollBackTransaction();
                LogHelper.Current.WriteException(ex);
                throw ex;
            }
            _con.CommitTransaction();

            LogHelper.Current.AddStr(
                String.Format(ResourceHelper.StrForLog[LogResKeys.DocTypeDeleted],
                    item.Name
                    ),
                _userInfo.Login
                );



        }

        public DocTypeItem CreateDocType(int folder, List<int> docTypeDrafts)
        {
            DocTypeItem item = _dtAccess.CreateDocType(folder);
            docTypeDrafts.Add(item.Id);

            LogHelper.Current.AddStr(
                String.Format(ResourceHelper.StrForLog[LogResKeys.DocTypeCreated],
                    item.Id
                    ),
                _userInfo.Login
                );

            return item;
        }

        public void SaveDocTypeItem(DocTypeItem item, List<int> docTypeDrafts)
        {
            if (item.IsNew)
            {
                docTypeDrafts.Remove(item.Id);
            }

            _con.BeginTransaction();
            try
            {
                _dtAccess.SaveDocTypeItem(item);
            }
            catch (Exception ex)
            {
                _con.RollBackTransaction();
                LogHelper.Current.WriteException(ex);
                throw ex;
            }
            _con.CommitTransaction();

            LogHelper.Current.AddStr(
               String.Format(ResourceHelper.StrForLog[LogResKeys.DocTypeSaved],
                   item.Name
                   ),
               _userInfo.Login
               );
        }

        /// <summary>
        /// was not tested
        /// </summary>
        /// <param name="docTypeId"></param>
        /// <param name="serviceUserInfo"></param>
        /// <param name="sk"></param>
        public virtual void ProcessDocCardsAfterRouteChange(
            DocTypeItem changedDocTypeItem, 
            UserInfo serviceUserInfo, 
            SettingKeeper sk,
            Notificator notificator
            )
        {
            DocCardData dcd = new DocCardData(_userInfo, _con, sk);
            DocCardMovementController cnt = new DocCardMovementController(serviceUserInfo, _con);
            EventData eventData = new EventData(_userInfo, _con);
            string[] fields = new string[]
            {
                DocCardFieldName.Id
            };
            
            DocCardFilterItem[] filterItems = new DocCardFilterItem[]
            {
                new DocCardFilterItem(DocCardFieldName.DocType, changedDocTypeItem.Id),
                new DocCardFilterItem(DocCardFieldName.DocStatus, DBConstKeeper.Current.DocStatus.UnderCons)
            };

            DataTable tbl = dcd.GetDocumentTable(fields, filterItems);
            RoutePoint[] newRoutePoints = changedDocTypeItem.RoutePoints.ToArray();
            int pointOfReturn = 0; //draft
            bool isAdmin = true, routeChange = true, loadRoute = true;
            
            List<long> documentDrafts = null;

            foreach (DataRow row in tbl.Rows)
            {
                DocCardItem docCardItem = dcd.LoadDocCard((long)row[DocCardFieldName.Id], loadRoute);
                if (cnt.CheckReturnDueToRouteChange(docCardItem, newRoutePoints))
                {
                    cnt.Reject(
                        docCardItem,
                        _userInfo.UserId,
                        pointOfReturn,
                        "",
                        isAdmin,
                        routeChange
                        );
                    
                    dcd.SaveDocCard(docCardItem, documentDrafts);

                    foreach (EventItem eventItem in cnt.Events)
                    {
                        eventData.AddEvent(eventItem);
                    }

                    foreach (NotificationItem notifItem in cnt.Notifications)
                    {
                        notificator.Notify(notifItem);
                    }    
                }
            }
        }

        public virtual DocTypeItem LoadDocTypeItem(int id, bool loadRoutePoints)
        {
            return _dtAccess.LoadDocTypeItem(id, loadRoutePoints);
        }

        public virtual int[] PrefixExists(string prefix)
        {
            return _dtAccess.PrefixExists(prefix);
        }

        public virtual DocTypeItem[] GetDocTypesWithCreatePermission(bool useTmpDocTypeSecurity)
        {
            return _dtAccess.GetDocTypesWithCreatePermission(useTmpDocTypeSecurity);
        }

        public virtual string GetNextInternalNumber(int docTypeId)
        {
            string nextNum = "";
            lock (_thisLock)
            {
                bool loadRoute = true;
                DocTypeItem item = _dtAccess.LoadDocTypeItem(docTypeId, !loadRoute);
                nextNum += item.Prefix;
                nextNum += item.NextNumber.ToString().PadLeft(6, '0');
                item.NextNumber++;
                _dtAccess.SaveDocTypeItem(item);
            }
            return nextNum;
        }
    }
}
