//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.MySqlDL
{
    public class AttachmentAccess
    {
        protected int _namingBase = 36;
        protected int _nameLength = 13;

        protected string ConvertToBase(long num, int nbase)
        {
            String chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

            // check if we can convert to another base
            if (nbase < 2 || nbase > chars.Length)
                return "";

            int r;
            String newNumber = "";

            // in r we have the offset of the char that was converted to the new base
            while (num >= nbase)
            {
                r = (int)(num % nbase);
                newNumber = chars[r] + newNumber;
                num = num / nbase;
            }
            // the last number to convert
            newNumber = chars[(int)num] + newNumber;

            return newNumber;
        }

        protected long ConvertFromBase(string num, int nbase)
        {
            String chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
            SortedList<char, int> charList = new SortedList<char, int>(chars.Length);
            for (int i = 0; i < chars.Length; i++)
            {
                charList.Add(chars[i], i);
            }

            long retVal = 0;
            for (int i = 0; i < num.Length; i++)
            {
                char ch = num[num.Length - i - 1];
                int val = charList[ch];
                retVal += (long)(Math.Pow(nbase, i) * val);
            }
            return retVal;
        }

        protected string GetNameById(long id)
        {
            return ConvertToBase(id, _namingBase).PadLeft(_nameLength, '0');
        }

        protected DBConnection _con;
        protected int _userId;
        protected SettingKeeper _sk;

        protected void CreateFolder(long folderId)
        {
            string newFolderPath = Path.Combine(_sk.AttachmentFolder, GetNameById(folderId));
            if (!Directory.Exists(newFolderPath))
            {
                Directory.CreateDirectory(newFolderPath);
            }
        }

        protected long GetNewAttachmentFolder()
        {
            long currentFolderId = 0;
            foreach (string folder in Directory.GetDirectories(_sk.AttachmentFolder))
            {
                long fldId = ConvertFromBase(Path.GetFileName(folder), _namingBase);
                if (fldId > currentFolderId)
                {
                    currentFolderId = fldId;
                }
            }

            long retVal = 0;
            bool isNew = false;
            if (currentFolderId > 0)
            {
                MySqlCommand cmd = _con.CreateCommand();
                cmd.CommandText = 
                    "select count(*) from Attachment where Folder=?Folder;";
                cmd.Parameters.Add("?Folder", MySqlDbType.Int32);
                cmd.Parameters["?Folder"].Value = currentFolderId;
                long filesInFolder = (long)cmd.ExecuteScalar();
                if (filesInFolder >= _sk.MaxFilesInFolder)
                {
                    retVal = currentFolderId + 1;
                    isNew = true;
                }
                else
                {
                    retVal = currentFolderId;
                }
            }
            else
            {
                retVal = 1;
                isNew = true;
            }

            if (isNew)
            {
                CreateFolder(retVal);
            }

            return retVal;
        }

        public AttachmentAccess(int userId, DBConnection con, SettingKeeper sk)
        {
            _con = con;
            _userId = userId;
            _sk = sk;
        }

        public AttachmentItem GetAttachmentItem(long id)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select Attachment.DocCard, Attachment.Name, Attachment.Folder, Attachment.File, " +
                "Attachment.ContentType, Attachment.Size, Attachment.UploadTime, " +
                "DocCard.DocType, DocCard.Folder as DocCardFolder, DocCard.DcUser " +
                "from Attachment inner join DocCard " +
                "on DocCard.DocCardId=Attachment.DocCard " +
                "where Attachment.AttachmentId=?Attachment;";

            cmd.Parameters.Add("?Attachment", MySqlDbType.Int64);
            cmd.Parameters["?Attachment"].Value = id;

            AttachmentItem retVal = null;

            int docType = 0, folder = 0, owner = 0;
            MySqlDataReader reader = cmd.ExecuteReader();
            if(reader.Read())
            {
                docType = reader.GetInt32("DocType");
                folder = reader.GetInt32("DocCardFolder");
                owner = reader.GetInt32("DcUser");
                retVal = new AttachmentItem(
                    id,
                    reader.GetInt64("DocCard"),
                    reader.GetString("Name"),
                    reader.GetInt64("Folder"),
                    reader.GetInt64("File"),
                    reader.GetString("ContentType"),
                    reader.GetInt32("Size"),
                    reader.GetDateTime("UploadTime")
                    );
            }
            reader.Close();

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            SecurityItem si = sa.GetFolderSecurity(folder);
            if (!si.Read && !(si.ReadOwn && owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.Folder,
                    ""
                    );
            }

            si = sa.GetDocTypeDocumentsSecurity(docType);
            if (!si.Read && !(si.ReadOwn && owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.DocType,
                    ""
                    );
            }

            return retVal;
        }
        
        public AttachmentItem[] GetDocCardAttachmentItems(long docCardId)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select DocType, Folder, DcUser, IsNew from DocCard where DocCardId=?DocCard";
            cmd.Parameters.Add("?DocCard", MySqlDbType.Int64);
            cmd.Parameters["?DocCard"].Value = docCardId;

            int docType = 0, folder = 0, owner = 0;
            bool isNew = false;
            MySqlDataReader reader = cmd.ExecuteReader();
            if (reader.Read())
            {
                isNew = reader.GetBoolean("IsNew");
                if (!isNew)
                {
                    docType = reader.GetInt32("DocType");
                }
                folder = reader.GetInt32("Folder");
                owner = reader.GetInt32("DcUser");
                
            }
            else
            {
                throw new ApplicationException("Document is not found!");
            }
            reader.Close();

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            SecurityItem si = sa.GetFolderSecurity(folder);
            if (!si.Read && !(si.ReadOwn && owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.Folder,
                    ""
                    );
            }

            if (!isNew)
            {
                si = sa.GetDocTypeDocumentsSecurity(docType);
                if (!si.Read && !(si.ReadOwn && owner == _userId))
                {
                    throw new DcUnathorizedAccessException(
                        DcUnathorizedAccessException.PolicyObjectType.DocType,
                        ""
                        );
                }
            }

            cmd.CommandText =
                "select Attachment.AttachmentId, Attachment.DocCard, Attachment.Name, " +
                "Attachment.Folder, Attachment.File, Attachment.ContentType, " +
                "Attachment.Size, Attachment.UploadTime, " +
                "DocCard.DocType, DocCard.Folder, DocCard.DcUser " +
                "from Attachment inner join DocCard " +
                "on DocCard.DocCardId=Attachment.DocCard " +
                "where Attachment.DocCard=?DocCard;";

            reader = cmd.ExecuteReader();

            List<AttachmentItem> itemList = new List<AttachmentItem>();

            while (reader.Read())
            {
                itemList.Add(
                    new AttachmentItem(
                    reader.GetInt64("AttachmentId"),
                    reader.GetInt64("DocCard"),
                    reader.GetString("Name"),
                    reader.GetInt64("Folder"),
                    reader.GetInt64("File"),
                    reader.GetString("ContentType"),
                    reader.GetInt32("Size"),
                    reader.GetDateTime("UploadTime")
                    )
                    );
            }
            reader.Close();

            return itemList.ToArray();
        }


        /// <summary>
        /// nb: save the DocCard before
        /// </summary>
        /// <param name="item"></param>
        /// <param name="inputStream"></param>
        public void SaveAttachment(AttachmentItem item, Stream inputStream)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            MySqlCommand cmd = _con.CreateCommand();

            cmd.Parameters.Add("?DocCard", MySqlDbType.Int64);
            cmd.Parameters.Add("?Name", MySqlDbType.String);
            cmd.Parameters.Add("?Folder", MySqlDbType.Int64);
            cmd.Parameters.Add("?File", MySqlDbType.Int64);
            cmd.Parameters.Add("?ContentType", MySqlDbType.String);
            cmd.Parameters.Add("?Size", MySqlDbType.Int32);
            cmd.Parameters.Add("?UploadTime", MySqlDbType.Datetime);
            cmd.Parameters.Add("?Attachment", MySqlDbType.Int64);

            cmd.Parameters["?DocCard"].Value = item.DocCardId;
            cmd.Parameters["?Name"].Value = item.FileName;
            cmd.Parameters["?ContentType"].Value = item.ContentType;
            cmd.Parameters["?Size"].Value = item.Size;
            cmd.Parameters["?UploadTime"].Value = item.UploadTime;

            // check security
            cmd.CommandText = "select DocType, Folder, DcUser from DocCard where DocCardId=?DocCard;";
            MySqlDataReader reader = cmd.ExecuteReader();
            int docType = 0, folder = 0, owner = 0;
            if (reader.Read())
            {
                docType = reader.GetInt32("DocType");
                folder = reader.GetInt32("Folder");
                owner = reader.GetInt32("DcUser");
            }
            reader.Close();

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            SecurityItem si = sa.GetFolderSecurity(folder);
            if (!si.Update && !(si.UpdateOwn && owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.Folder,
                    ""
                    );
            }

            si = sa.GetDocTypeDocumentsSecurity(docType);
            if (!si.Update && !(si.UpdateOwn && owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.DocType,
                    ""
                    );
            }

            bool isNew = false;
            if (item.Id == null)
            {
                isNew = true;
                item.Folder = GetNewAttachmentFolder();
                cmd.Parameters["?Folder"].Value = item.Folder;

                cmd.CommandText =
                    "insert into attachment (DocCard, Name, Folder, ContentType, Size, UploadTime) " +
                    "values (?DocCard, ?Name, ?Folder, ?ContentType, ?Size, ?UploadTime);" +
                    "set @lastId:=last_insert_id(); " +
                    "update attachment set File=@lastId where AttachmentId=@lastId;";
                cmd.ExecuteNonQuery();
                cmd.CommandText = "select cast(@lastId as unsigned);";
                reader = cmd.ExecuteReader();
                if (reader.Read())
                {
                    item.Id = reader.GetInt64(0);
                    item.File = item.Id;
                }
                reader.Close();
            }
            else
            {
                cmd.Parameters["?Folder"].Value = item.Folder;
                cmd.Parameters["?File"].Value = item.File;
                cmd.Parameters["?Attachment"].Value = item.Id;
                cmd.CommandText =
                    "update attachment set DocCard=?DocCard, Name=?Name, ContentType=?ContentType, " +
                    "Size=?Size, UploadTime=?UploadTime where AttachmentId=?Attachment;";
                cmd.ExecuteNonQuery();
            }

            string filePath = Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));
            filePath = Path.Combine(filePath, GetNameById(item.File.Value) + Path.GetExtension(item.FileName));
            if (File.Exists(filePath) && isNew)
            {
                throw new ApplicationException(
                    String.Format("File {0} already exists!", filePath)
                    );
            }

            Console.WriteLine(filePath);
            FileStream fs = File.Create(filePath, _sk.AttachmentBufferSize);
            byte[] buffer = new byte[_sk.AttachmentBufferSize];
            int readBytes = 0;
            while ((readBytes = inputStream.Read(buffer, 0, _sk.AttachmentBufferSize)) > 0)
            {
                fs.Write(buffer, 0, readBytes);
            }
            fs.Close();
        }

        public Stream OpenAttachment(AttachmentItem item)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }


            string folderPath = Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));
            
            string filePath = GetNameById(item.File.Value);
            filePath += Path.GetExtension(item.FileName);
            filePath = 
                Path.Combine(folderPath, filePath);

            if (!File.Exists(filePath))
            {
                throw new FileNotFoundException(
                    String.Format("Attachment file '{0}' is not found.", filePath), 
                    filePath
                    );
            }

            return File.OpenRead(filePath);
        }

        public void DeleteAttachment(AttachmentItem item)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.Parameters.Add("?DocCard", MySqlDbType.Int64);
            cmd.Parameters["?DocCard"].Value = item.DocCardId;

            cmd.CommandText = "select DocType, Folder, DcUser from DocCard where DocCardId=?DocCard;";
            MySqlDataReader reader = cmd.ExecuteReader();
            int docType = 0, folder = 0, owner = 0;
            if (reader.Read())
            {
                docType = reader.GetInt32("DocType");
                folder = reader.GetInt32("Folder");
                owner = reader.GetInt32("DcUser");
            }
            reader.Close();

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            SecurityItem si = sa.GetFolderSecurity(folder);
            if (!si.Update && !(si.UpdateOwn && owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.Folder,
                    ""
                    );
            }

            si = sa.GetDocTypeDocumentsSecurity(docType);
            if (!si.Update && !(si.UpdateOwn && owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.DocType,
                    ""
                    );
            }

            cmd.CommandText = "delete from attachment where AttachmentId=?Attachment";
            cmd.Parameters.Add("?Attachment", MySqlDbType.Int64);
            cmd.Parameters["?Attachment"].Value = item.Id;
            cmd.ExecuteNonQuery();

            string filePath = Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));
            filePath = Path.Combine(filePath, GetNameById(item.File.Value) + Path.GetExtension(item.FileName));
            if (File.Exists(filePath))
            {
                File.Delete(filePath);
            }

            //if (Directory.GetFiles(Path.GetDirectoryName(filePath)).Length == 0)
            //{
            //    Directory.Delete(Path.GetDirectoryName(filePath));
            //}
        }

    }
}
