//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.MySqlDL
{
    public class DocCardReferenceAccess
    {
        protected int _userId;
        protected DBConnection _con;
        protected SettingKeeper _sk;

        protected class Tbl : DataTbl
        {
            public DataColumn MasterDocCard = new DataColumn("MasterDocCard", MySqlDbType.Int64);
            public DataColumn SlaveDocCard = new DataColumn("SlaveDocCard", MySqlDbType.Int64);
            public DataColumn DocCardReferenceId = new DataColumn(
                "DocCardReferenceId",
                MySqlDbType.Int64,
                true,
                false
                );
            public Tbl()
            {
                _items = new DataColumn[]
                {
                    MasterDocCard,
                    SlaveDocCard,
                    DocCardReferenceId
                };
            }
        }

        protected Tbl _tbl = new Tbl();

        public DocCardReferenceAccess(int userId, DBConnection con, SettingKeeper sk)
        {
            _userId = userId;
            _con = con;
            _sk = sk;
        }

        public DocCardReferenceItem[] GetDocCardReferences(long docCardId)
        {
            // if access is denied, it throws the unauthorized access exception
            bool loadRoute = true;
            DocCardItem docCardItem = (new DocCardAccess(_userId, _con, _sk)).LoadDocCard(docCardId, !loadRoute);

            MySqlCommand cmd = _con.CreateCommand();
            StringBuilder sb = new StringBuilder();
            foreach (DataColumn col in _tbl.Items)
            {
                if (sb.Length != 0)
                {
                    sb.Append(", ");
                }
                sb.Append(col.Name);
            }

            cmd.CommandText =
                String.Format(
                    "select {0} from DocCardReference where MasterDocCard=?MasterDocCard;",
                    sb.ToString()
                    );
            
            _tbl.MasterDocCard.SetParameter(cmd, docCardId);

            MySqlDataReader reader = cmd.ExecuteReader();

            List<DocCardReferenceItem> refs = new List<DocCardReferenceItem>();

            while (reader.Read())
            {
                refs.Add(
                    new DocCardReferenceItem(
                        reader.GetInt64(_tbl.MasterDocCard.Name),
                        reader.GetInt64(_tbl.SlaveDocCard.Name),
                        reader.GetInt64(_tbl.DocCardReferenceId.Name)
                        )
                    );
            }

            reader.Close();

            return refs.ToArray();
        }

        public void SaveDocCardReferences(long docCardId, DocCardReferenceItem[] references)
        {
            
            long master = docCardId;
            bool loadRoute = true;
            DocCardItem docCardItem =
                (new DocCardAccess(_userId, _con, _sk)).LoadDocCard(master, !loadRoute);
            SecurityAccess sa = new SecurityAccess(_userId, _con);
            if (!sa.GetDocumentCardSecurity(docCardItem).Update)
            {
                throw new DcUnathorizedAccessException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            _tbl.MasterDocCard.SetParameter(cmd, master);

            if (references.Length > 0)
            {
                List<long> existentIds = new List<long>(references.Length);

                StringBuilder sb = null;

                foreach (DocCardReferenceItem item in references)
                {
                    _tbl.SlaveDocCard.SetParameter(cmd, item.Reference);

                    if (item.Id == null)
                    {
                        StringBuilder colSb = new StringBuilder();
                        StringBuilder valSb = new StringBuilder();
                        foreach (DataColumn col in _tbl.Items)
                        {
                            if (!col.AutoInc)
                            {
                                if (colSb.Length != 0)
                                {
                                    colSb.Append(", ");
                                }
                                if (valSb.Length != 0)
                                {
                                    valSb.Append(", ");
                                }

                                colSb.Append(col.Name);
                                valSb.AppendFormat("?{0}", col.Name);
                            }
                        }

                        cmd.CommandText =
                            String.Format(
                                "insert into DocCardReference ({0}) Values ({1});",
                                colSb.ToString(),
                                valSb.ToString()
                                );
                        cmd.ExecuteNonQuery();
                        item.Id = cmd.LastInsertedId;

                    }
                    else
                    {
                        sb = new StringBuilder();
                        foreach (DataColumn col in _tbl.Items)
                        {
                            if (!col.AutoInc)
                            {
                                if (sb.Length != 0)
                                {
                                    sb.Append(", ");
                                }
                                sb.AppendFormat("{0}=?{0}", col.Name);
                            }
                        }

                        cmd.CommandText = String.Format(
                            "update DocCardReference set {0} where {1}=?{1};",
                            sb.ToString(),
                            _tbl.DocCardReferenceId.Name
                            );

                        _tbl.DocCardReferenceId.SetParameter(cmd, item.Id.Value);
                        cmd.ExecuteNonQuery();
                    }
                    existentIds.Add(item.Id.Value);
                }

                sb = new StringBuilder();
                foreach (long existentId in existentIds)
                {
                    if (sb.Length != 0)
                    {
                        sb.Append(", ");
                    }
                    sb.Append(existentId.ToString());
                }

                cmd.CommandText = String.Format(
                    "delete from DocCardReference where {0} not in ({1}) and {2}=?{2}",
                    _tbl.DocCardReferenceId.Name,
                    sb.ToString(),
                    _tbl.MasterDocCard.Name
                    );

                cmd.ExecuteNonQuery();
            }
            else
            {
                cmd.CommandText = String.Format(
                    "delete from DocCardReference where {0}=?{0};",
                    _tbl.MasterDocCard.Name
                    );
                cmd.ExecuteNonQuery();
            }
        }

        public void DeleteDocCardReferences(long docCardId)
        {
            bool loadRoute = true;
            DocCardItem docCardItem =
                     (new DocCardAccess(_userId, _con, _sk)).LoadDocCard(docCardId, !loadRoute);
            SecurityAccess sa = new SecurityAccess(_userId, _con);
            if (!sa.GetDocumentCardSecurity(docCardItem).Update)
            {
                throw new DcUnathorizedAccessException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            _tbl.MasterDocCard.SetParameter(cmd, docCardId);

            cmd.CommandText = String.Format(
                "delete from DocCardReference where {0}=?{0};",
                _tbl.MasterDocCard.Name
                );

            cmd.ExecuteNonQuery();
        }

    }
}
