//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.MySqlDL
{
    public class DocTypeAccess
    {
        private DBConnection _con;
        private int _userId;

        /// <summary>
        /// for unit testing
        /// </summary>
        public DocTypeAccess() { }

        public DocTypeAccess(int userId, DBConnection con)
        {
            _con = con;
            _userId = userId;
        }

        public DocTypeItem CreateDocType(int folder)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            int? id = null;

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            if (!sa.GetFolderSecurity(folder).Create)
            {
                throw new DcUnathorizedAccessException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "insert into DocType (DocTypeId, Folder, IsNew, DcUser) Values (default, ?Folder, true, ?DcUser);";
            cmd.Parameters.Add("?Folder", MySqlDbType.Int32);
            cmd.Parameters["?Folder"].Value = folder;
            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters["?DcUser"].Value = _userId;
            cmd.ExecuteNonQuery();
            id = (int)cmd.LastInsertedId;

            cmd.Parameters.Add("?DocType", MySqlDbType.Int32);
            cmd.Parameters["?DocType"].Value = id.Value;
            cmd.CommandText =
                "insert into DocTypePermission (DocType, DcRole, C, R, U, D, RO, UO, DO) " +
                "select ?DocType, DcRole, C, R, U, D, RO, UO, DO from NewDocTypePermission;";
            cmd.ExecuteNonQuery();


            DocTypeItem docType = new DocTypeItem(
                id.Value,
                folder,
                "",
                "",
                "",
                1,
                _userId
                );
            docType.IsNew = true;
            docType.SetRoutePointsLoaded(true);

            return docType;
        }

        public virtual DocTypeItem LoadDocTypeItem(int id, bool loadRoutePoints)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }


            DocTypeItem item = new DocTypeItem();

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select Folder, Name, Description, Prefix, NextNumber, DcUser from DocType where DocTypeId=?id";

            cmd.Parameters.Add("?id", MySqlDbType.Int32);
            cmd.Parameters["?id"].Value = id;

            MySqlDataReader reader = cmd.ExecuteReader();
            bool found = false;
            while (reader.Read())
            {
                found = true;
                item.Id = id;
                item.Folder = reader.GetInt32("Folder");
                item.Name = reader.GetString("Name");
                item.Description = reader.GetString("Description");
                item.Prefix = reader.GetString("Prefix");
                item.NextNumber = reader.GetInt32("NextNumber");
                item.Owner = reader.GetInt32("DcUser");
            }
            reader.Close();

            if (!found)
            {
                throw new DcObjectNotFoundException();
            }

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            SecurityItem si = sa.GetFolderSecurity(item.Folder);
            if (!si.Read && !(si.ReadOwn && item.Owner == _userId) )
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.Folder,
                    String.Format("Folder id={0}",item.Folder)
                    );
            }

            if (loadRoutePoints)
            {
                LoadDocTypeRoutePoints(item);
            }


            return item;
        }

        public void LoadDocTypeRoutePoints(DocTypeItem item)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }
            
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText =
                "select RoutePointId, DocType, OrderNum, Expression from RoutePoint where DocType=?DocType order by OrderNum;";

            cmd.Parameters.Add("?DocType", MySqlDbType.Int32);
            cmd.Parameters["?DocType"].Value = item.Id;

            item.RoutePoints = new List<RoutePoint>();
            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                RoutePoint rp = new RoutePoint(
                    reader.GetInt32("RoutePointId"),
                    reader.GetString("Expression"),
                    reader.GetInt32("OrderNum")
                    );

                item.RoutePoints.Add(rp);
            }
            reader.Close();

            cmd.CommandText = "select ri.RouteItemId, ri.DocType, ri.DcUser, " +
                "ri.RoutePoint, ri.RouteItemType, rp.OrderNum " +
                "from RouteItem as ri inner join RoutePoint as rp on ri.RoutePoint=rp.RoutePointId " +
                "where ri.DocType=?DocType order by rp.OrderNum, ri.ItemIndex";
            reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                RouteItem ri = new RouteItem(
                    reader.GetInt32("RouteItemId"),
                    reader.GetInt32("DcUser"),
                    reader.GetInt32("RouteItemType")
                    );
                item.RoutePoints[reader.GetInt32("OrderNum") - 1].RouteItems.Add(ri);
            }
            reader.Close();
            
        }

        public virtual void SaveDocTypeItem(DocTypeItem item)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            SecurityItem si = sa.GetFolderSecurity(item.Folder);

            if (!si.Update && !(si.UpdateOwn && item.Owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.Folder,
                    String.Format("Access denied due to the item folder policy. Folder={0}!", item.Folder)
                    );
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "update DocType set Name=?Name, Folder=?Folder, Description=?Description, " +
                "Prefix=?Prefix, NextNumber=?NextNumber, IsNew=?IsNew " +
                "where DocTypeId=?DocTypeId";

            //MySqlDataReader reader = null;
            cmd.Parameters.Add("?Name", MySqlDbType.String);
            cmd.Parameters["?Name"].Value = item.Name;

            cmd.Parameters.Add("?Prefix", MySqlDbType.String);
            cmd.Parameters["?Prefix"].Value = item.Prefix;

            cmd.Parameters.Add("?Description", MySqlDbType.String);
            cmd.Parameters["?Description"].Value = item.Description;

            cmd.Parameters.Add("?NextNumber", MySqlDbType.Int32);
            cmd.Parameters["?NextNumber"].Value = item.NextNumber;

            cmd.Parameters.Add("?Folder", MySqlDbType.Int32);
            cmd.Parameters["?Folder"].Value = item.Folder;

            cmd.Parameters.Add("?IsNew", MySqlDbType.Bit);
            cmd.Parameters["?IsNew"].Value = false;

            cmd.Parameters.Add("?DocTypeId", MySqlDbType.Int32);
            cmd.Parameters["?DocTypeId"].Value = item.Id;

            cmd.ExecuteNonQuery();

            item.SetSavedState();

            if (item.RoutePointsLoaded)
            {

                MySqlCommand pointCmd = _con.CreateCommand();
                pointCmd.Parameters.Add("?RoutePointId", MySqlDbType.Int32);
                pointCmd.Parameters.Add("?DocType", MySqlDbType.Int32);
                pointCmd.Parameters.Add("?OrderNum", MySqlDbType.Int32);
                pointCmd.Parameters.Add("?Expression", MySqlDbType.String);
                pointCmd.Parameters["?DocType"].Value = item.Id;

                MySqlCommand itemCmd = _con.CreateCommand();
                itemCmd.Connection = _con.MySqlConnection;
                itemCmd.Parameters.Add("?DocType", MySqlDbType.Int32);
                itemCmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
                itemCmd.Parameters.Add("?RoutePoint", MySqlDbType.Int32);
                itemCmd.Parameters.Add("?RouteItemId", MySqlDbType.Int32);
                itemCmd.Parameters.Add("?RouteItemType", MySqlDbType.Int32);
                itemCmd.Parameters.Add("?ItemIndex", MySqlDbType.Int32);
                itemCmd.Parameters["?DocType"].Value = item.Id;

                cmd = _con.CreateCommand();
                cmd.CommandText = 
                    "create temporary table if not exists tmpPointId(RoutePoint int);" +
                    "delete from tmpPointId;" +
                    "create temporary table if not exists tmpItemId(RouteItem int);" +
                    "delete from tmpItemId;";
                cmd.ExecuteNonQuery();

                for (int p = 0; p < item.RoutePoints.Count; p++)
                {
                    RoutePoint rp = item.RoutePoints[p];
                    pointCmd.Parameters["?OrderNum"].Value = rp.OrderNum;
                    pointCmd.Parameters["?Expression"].Value = rp.Expression;
                    if (rp.Id == null)
                    {
                        pointCmd.CommandText =
                            "insert into RoutePoint (DocType, OrderNum, Expression) " +
                            "Values (?DocType, ?OrderNum, ?Expression);";
                        pointCmd.ExecuteNonQuery();
                        rp.Id = (int)pointCmd.LastInsertedId;
                        pointCmd.CommandText = "insert into tmpPointId Values (last_insert_id());";
                        pointCmd.ExecuteNonQuery();
                    }
                    else
                    {
                        pointCmd.Parameters["?RoutePointId"].Value = rp.Id;
                        pointCmd.CommandText =
                            "update RoutePoint set OrderNum=?OrderNum, Expression=?Expression " +
                            "where RoutePointId=?RoutePointId;" +
                            "insert into tmpPointId Values (?RoutePointId);";
                        pointCmd.ExecuteNonQuery();
                    }
                    

                    itemCmd.Parameters["?RoutePoint"].Value = rp.Id;
                    for (int i = 0; i < rp.RouteItems.Count; i++)
                    {
                        RouteItem ri = rp.RouteItems[i];
                        itemCmd.Parameters["?DcUser"].Value = ri.User;
                        itemCmd.Parameters["?RouteItemType"].Value = ri.RouteItemType;
                        itemCmd.Parameters["?ItemIndex"].Value = i;
                        if (ri.Id == null)
                        {
                            itemCmd.CommandText =
                                "insert into RouteItem (DocType, DcUser, RoutePoint, RouteItemType, ItemIndex) " +
                                "Values (?DocType, ?DcUser, ?RoutePoint, ?RouteItemType, ?ItemIndex);";
                            itemCmd.ExecuteNonQuery();
                            ri.Id = (int)itemCmd.LastInsertedId;
                            itemCmd.CommandText = "insert into tmpItemId Values (last_insert_id());";
                            itemCmd.ExecuteNonQuery();
                            
                            
                        }
                        else
                        {
                            itemCmd.Parameters["?RouteItemId"].Value = ri.Id;
                            itemCmd.CommandText =
                                "update RouteItem Set DcUser=?DcUser, RoutePoint=?RoutePoint, " +
                                "RouteItemType=?RouteItemType, ItemIndex=?ItemIndex where RouteItemId=?RouteItemId;" +
                                "insert into tmpItemId Values (?RouteItemId);";
                            itemCmd.ExecuteNonQuery();
                        }
                    }
                }
                
                cmd.Parameters.Add("?DocType", MySqlDbType.Int32);
                cmd.Parameters["?DocType"].Value = item.Id;
                cmd.CommandText = 
                    "delete from DocCardRouteItem " +
                    "where DocCard in (select DocCardId from DocCard where DocType=?DocType) and " +
                    " RouteItem not in (select RouteItem from tmpItemId);" +
                    "delete from RouteItem " +
                    "where DocType=?DocType and RouteItemId not in (select RouteItem from tmpItemId);" +
                    "delete from RoutePoint " +
                    "where DocType=?DocType and RoutePointId not in (select RoutePoint from tmpPointId);";
                    
                cmd.ExecuteNonQuery();
            }

        }

        public DocTypeItem[] GetDocTypes(bool useTmpFolderSecurityTable)
        {
            return GetDocTypes(null, useTmpFolderSecurityTable);
        }

        public DocTypeItem[] GetDocTypes(int? folder)
        {
            return GetDocTypes(folder, false);
        }

        public DocTypeItem[] GetDocTypes()
        {
            return GetDocTypes(null, false);
        }

        public DocTypeItem[] GetDocTypes(int? folder, bool useTmpFolderSecurityTable)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            if (!useTmpFolderSecurityTable)
            {
                SecurityAccess sa = new SecurityAccess(_userId, _con);
                sa.CreateFolderSecurityTable();
            }

            List<DocTypeItem> list = new List<DocTypeItem>();

            MySqlCommand cmd = _con.CreateCommand();

            StringBuilder cmdStr = null;
            cmdStr = new StringBuilder(
                    "select DocTypeId, DocType.Name, Prefix, Description, NextNumber, DocType.Folder, DocType.DcUser " +
                    "from DocType " +
                    "inner join tmpFolderSecurity " +
                    "on DocType.Folder=tmpFolderSecurity.Folder and tmpFolderSecurity.DcUser=?DcUser and " +
                    "(tmpFolderSecurity.R or (tmpFolderSecurity.RO and DocType.DcUser=?DcUser)) " +
                    "where not IsNew"
                    );

            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters["?DcUser"].Value = _userId;

            if (folder != null)
            {
                cmdStr.Append(" and DocType.Folder=?Folder");
                cmd.Parameters.Add("?Folder", MySqlDbType.Int32);
                cmd.Parameters["?Folder"].Value = folder.Value;
            }

            cmd.CommandText = cmdStr.ToString();

            MySqlDataReader reader = cmd.ExecuteReader();

            while (reader.Read())
            {
                DocTypeItem item = new DocTypeItem(
                    reader.GetInt32("DocTypeId"),
                    reader.GetInt32("Folder"),
                    reader.GetString("Name"),
                    reader.GetString("Prefix"),
                    reader.GetString("Description"),
                    reader.GetInt32("NextNumber"),
                    reader.GetInt32("DcUser")
                    );

                list.Add(item);
            }
            reader.Close();

            return list.ToArray();
        }

        public DocTypeItem[] GetDocTypesWithCreatePermission(bool useTmpDocTypeSecurityTable)
        {

            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            if (!useTmpDocTypeSecurityTable)
            {
                SecurityAccess sa = new SecurityAccess(_userId, _con);
                sa.CreateDocTypeSecurityTable();
            }

            List<DocTypeItem> list = new List<DocTypeItem>();

            MySqlCommand cmd = _con.CreateCommand();

            StringBuilder cmdStr = null;
            cmdStr = new StringBuilder(
                    "select DocTypeId, DocType.Name, Prefix, Description, NextNumber, DocType.Folder, DocType.DcUser " +
                    "from DocType " +
                    "inner join tmpDocTypeSecurity " +
                    "on DocType.DocTypeId=tmpDocTypeSecurity.DocType and tmpDocTypeSecurity.DcUser=?DcUser " +
                    "and tmpDocTypeSecurity.C " +
                    "where not IsNew order by DocType.Name;"
                    );

            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters["?DcUser"].Value = _userId;

            cmd.CommandText = cmdStr.ToString();

            MySqlDataReader reader = cmd.ExecuteReader();

            while (reader.Read())
            {
                DocTypeItem item = new DocTypeItem(
                    reader.GetInt32("DocTypeId"),
                    reader.GetInt32("Folder"),
                    reader.GetString("Name"),
                    reader.GetString("Prefix"),
                    reader.GetString("Description"),
                    reader.GetInt32("NextNumber"),
                    reader.GetInt32("DcUser")
                    );

                list.Add(item);
            }
            reader.Close();

            return list.ToArray();
        }

        //public DocTypeToken[] GetDocTypeList()
        //{
        //    return GetDocTypeList(null);
        //}


        //public DocTypeToken[] GetDocTypeList(int? folder)
        //{
        //    if (!_con.IsOpen)
        //    {
        //        throw new DcConnectionClosedException();
        //    }

        //    string cmdStr = "select DocTypeId, Name from DocType where not IsNew";
        //    MySqlCommand cmd = new MySqlCommand();
        //    cmd.Connection = _con.MySqlConnection;
        //    if (folder != null)
        //    {
        //        cmd.CommandText = cmdStr + " && Folder=?Folder";
        //        cmd.Parameters.Add("?Folder", MySqlDbType.Int32);
        //        cmd.Parameters["?Folder"].Value = folder;
        //    }
        //    else
        //    {
        //        cmd.CommandText = cmdStr;
        //    }

        //    MySqlDataReader reader = cmd.ExecuteReader();

        //    List<DocTypeToken> arr = new List<DocTypeToken>();
        //    while (reader.Read())
        //    {
        //        DocTypeToken dtt = new DocTypeToken(
        //            reader.GetInt32("DocTypeId"),
        //            reader.GetString("Name")
        //            );
        //        arr.Add(dtt);
        //    }
        //    reader.Close();
        //    return arr.ToArray();
        //}

        public void DeleteDocType(int id)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText =
                "select Folder, DcUser from DocType where DocTypeId=?DocType;";

            cmd.Parameters.Add("?DocType", MySqlDbType.Int32);
            cmd.Parameters["?DocType"].Value = id;

            MySqlDataReader reader = cmd.ExecuteReader();
            bool isRead = reader.Read();
            if (!isRead)
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
                throw new DcObjectNotFoundException();
            }
            int folder = reader.GetInt32("Folder"), owner = reader.GetInt32("DcUser");
            reader.Close();

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            SecurityItem si = sa.GetFolderSecurity(folder);
            if (!si.Delete && !(si.DeleteOwn && owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.Folder,
                    String.Format("User {0} doesn't have delete right! DocTypeId={1}", _userId, id)
                    );
            }
                        
            cmd.CommandText = 
                "update dcuser set LastDocType=NULL where LastDocType=?DocType;" +
                "delete from RouteItem where DocType=?DocType;" +
                "delete from RoutePoint where DocType=?DocType;" +
                "delete from DocTypePermission where DocType=?DocType;" +
                "delete from DocType where DocTypeId=?DocType;";
            cmd.ExecuteNonQuery();

        }

        
        public int[] PrefixExists(string prefix)
        {
            prefix = prefix.ToUpper();
            List<int> retVal = new List<int>();
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select DocTypeId from DocType where UPPER(Prefix)=?Prefix and not IsNew";
            cmd.Parameters.Add("?Prefix", MySqlDbType.String);
            cmd.Parameters["?Prefix"].Value = prefix;
            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                retVal.Add(reader.GetInt32("DocTypeId"));
            }
            reader.Close();

            return retVal.ToArray();
        }
       
        
    }

}