//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Resources;
using NUnit.Framework;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.Messaging;


namespace Evpo.DocCircUnitTest
{


    [TestFixture]
    public class MessageCreatorTest
    {

        protected struct FixtureConst
        {
            public const string ServerName = "server-tst";
            public const string ApplicationUrl = "http://server.TEST.ORG/EvpoDc";
            public const int DocCircUser1 = 10;
            public const int DocCircUser2 = 11;
            public const int ServiceUserId = 12;
            public const string DocCircUser2Email = "doccircuser2@test.org";
            public const string DocCircUser2Login = "DocCircUser2";
            public const string ServiceUserEmail = "doccircservice@test.org";
            public const string ServiceUserFullName = "DocCirc service";
            public const string DocCircUser1FullName = "DocCirc user1";
            public const bool DocCircUser2InstantMessaging = true;
            public const string DocCircUser2ImAddress = "doccircuser2@im.test.org";
            public const long DocCard1 = 21, DocCard2 = 22;
            public const int DocCard1RoutePointNum = 1;
            public const int DocType1 = 23;
            public const string DocType1Name = "Doc type 1";
            public const long UserNotification1 = 31;
        }

        protected struct ResourceString
        {
            public const string NextDecisionNotificationType = "next decision";
            public const string UnderConsDocStatus = "under consideration";
        }

        protected struct DocCard1Property
        {
            public const string Subject = "doc card 1 subject";
            public const string InternalNum = "doc card 1 internal num";
            public const long Id = FixtureConst.DocCard1;
            public const int RegisteringUser = FixtureConst.DocCircUser1;
        }

        protected class MockUserInfo : UserInfo
        {
            public void SetEmails(string[] emails)
            {
                _emails = emails;
            }

            public void SetUserProperties(UserProperties props)
            {
                _userProperties = props;
            }
        }

        protected class MockUserHelperInstance : UserHelperInstance
        {
            public override string[] GetUserEmails(int userId, DataConnection con)
            {
                string[] retVal = null;
                if (userId == FixtureConst.DocCircUser2)
                {
                    retVal = new string[] { FixtureConst.DocCircUser2Email };    
                }
                return retVal;
            }

            public override UserProperties GetUserProperties(int userId, DataConnection con)
            {
                UserProperties props = new UserProperties();
                if (userId == FixtureConst.DocCircUser1)
                {
                    props.FullName = FixtureConst.DocCircUser1FullName;
                    
                }
                else if (userId == FixtureConst.DocCircUser2)
                {
                    props.InstantMessaging = FixtureConst.DocCircUser2InstantMessaging;
                    props.ImAddress = FixtureConst.DocCircUser2ImAddress;
                    props.Login = FixtureConst.DocCircUser2Login;
                }
                else
                {
                    throw new ApplicationException("Unknown doc circ user for this unit test!");
                }
                return props;
            }
	
        }

        protected class MockStringResHelperInstance : StringResHelper
        {
            public override string this[string key]
            {
                get
                {
                    string retVal = null;
                    if (key == UIResKeys.NextDecisionNotificationType)
                    {
                        retVal = ResourceString.NextDecisionNotificationType;
                    }
                    else if (key == UIResKeys.UnderConsDocStatus)
                    {
                        retVal = ResourceString.UnderConsDocStatus;
                    }
                    else
                    {
                        throw new ApplicationException("Resource is unknown in this test!");
                    }
                    return retVal;
                }
            }
        }

        protected class MockDocCardData : DocCardData
        {
            public override DocCardItem LoadDocCard(long id, bool loadRoute)
            {
                DocCardItem item = null;
                if (id == FixtureConst.DocCard1)
                {
                    item = new DocCardItem();
                    item.Id = id;
                    item.Subject = DocCard1Property.Subject;
                    item.InternalNumber = DocCard1Property.InternalNum;
                    item.RegisteringUser = DocCard1Property.RegisteringUser;
                    item.DocType = FixtureConst.DocType1;
                    item.DocStatus = DBConstKeeper.Current.DocStatus.UnderCons;
                    item.RoutePointNum = FixtureConst.DocCard1RoutePointNum;
                }
                else
                {
                    throw new ApplicationException("DocCardId is unknown for this test!");
                }
                return item;
            }
        }

        protected class MockDocTypeData : DocTypeData
        {
            public override DocTypeItem LoadDocTypeItem(int id, bool loadRoutePoints)
            {
                DocTypeItem retVal = new DocTypeItem();
                if (id == FixtureConst.DocType1)
                {
                    retVal.Id = id;
                    retVal.Name = FixtureConst.DocType1Name;
                }
                return retVal;
            }
        }


        protected class MockResourceManager : ResourceManager
        {
            public override string GetString(string name)
            {
                string retVal = null;
                if (name == UIResKeys.NextDecisionNotificationType)
                {
                    retVal = ResourceString.NextDecisionNotificationType;
                }
                else if (name == UIResKeys.UnderConsDocStatus)
                {
                    retVal = ResourceString.UnderConsDocStatus;
                }
                else
                {
                    throw new ApplicationException("Resource is unknown in this test!");
                }
                return retVal;
            }
        }

        protected class TestMessageCreator : MessageCreator
        {
            public TestMessageCreator(DataConnection con, UserInfo serviceUser, SettingKeeper sk)
                : base(con, serviceUser, sk)
            {
            }

            protected override DocCardData GetDocCardData(UserInfo userInfo, DataConnection con, SettingKeeper sk)
            {
                return new MockDocCardData();
            }

            protected override DocTypeData GetDocTypeData(UserInfo userInfo, DataConnection con)
            {
                return new MockDocTypeData();
            }

            protected override System.Resources.ResourceManager GetResourceManager()
            {
                return new MockResourceManager();
            }
        }

        


        protected MockUserInfo _serviceUser;
        protected SettingKeeper _sk;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            _serviceUser = new MockUserInfo();
            _serviceUser.SetEmails(new string[] { FixtureConst.ServiceUserEmail });
            
            UserProperties userProp = new UserProperties();
            userProp.UserId = FixtureConst.ServiceUserId;
            userProp.Login = "doccircservice";
            userProp.FullName = FixtureConst.ServiceUserFullName;
            userProp.AutomaticPass = false;
            userProp.AllowDeputyNotification = true;
            userProp.Department = 0;
            userProp.InstantMessaging = false;
            userProp.ImAddress = "";

            _serviceUser.SetUserProperties(userProp);

            _sk = new SettingKeeper();
            //_sk.ServiceUserEmail = FixtureConst.ServiceUserEmail;
            _sk.ServerName = FixtureConst.ServerName;
            _sk.ApplicationUrl = FixtureConst.ApplicationUrl;

            DBConstKeeper.Current = new DBConstKeeper();
            DBConstKeeper.Current.NotificationType = new NotificationTypeDirectory(1, 2, 3, 4, 5, 6, 7, 8, 9);
            DBConstKeeper.Current.DocStatus = new DocStatusDirectory(1, 2, 3, 4);

            UserHelper.Current = new MockUserHelperInstance();
            ResourceHelper.StrForUI = new MockStringResHelperInstance();
        }

        [Test]
        public void CreateNotificationMessage()
        {
            TestMessageCreator mc = new TestMessageCreator(null, _serviceUser, _sk);

            NotificationItem notifItem = new NotificationItem(
                FixtureConst.UserNotification1,
                FixtureConst.DocCard1,
                DBConstKeeper.Current.NotificationType.NextDecision,
                FixtureConst.DocCircUser2,
                new DateTime(2007, 06, 07, 16, 13, 00),
                "text1",
                "Hi there1!",
                FixtureConst.DocCircUser1,
                FixtureConst.DocCard1RoutePointNum
                );

            NotificationMessage msg = mc.CreateNotificationMessage(notifItem); // tested method
            
            Assert.IsNotNull(msg);

            Assert.AreEqual(FixtureConst.UserNotification1, msg.UserNotificationId);
            Assert.AreEqual(1, msg.Recipients.Length);
            Assert.AreEqual(FixtureConst.DocCircUser2Email, msg.Recipients[0]);
            Assert.AreEqual(FixtureConst.DocCircUser2Login, msg.UserLogin);
            Assert.AreEqual(FixtureConst.ServiceUserEmail, msg.ServiceUserEmail);

            //Assert.IsNotNull(msg.ServiceUserProp);
            //Assert.AreEqual(FixtureConst.ServiceUserId, msg.ServiceUserProp.UserId);
            Assert.AreEqual(FixtureConst.ServiceUserFullName, msg.ServiceUserDisplayName);
            Assert.AreEqual(ResourceString.NextDecisionNotificationType, msg.NotificationTypeText);
            Assert.AreEqual(DocCard1Property.Subject, msg.DocCardSubject);
            Assert.AreEqual(DocCard1Property.InternalNum, msg.InternalNum);
            Assert.AreEqual(
                String.Format(
                    "{0}/DocumentCard.aspx?id={1}",
                    FixtureConst.ApplicationUrl,
                    DocCard1Property.Id),
                msg.DocCardUrl
                );

            Assert.AreEqual("text1", msg.Text);
            Assert.AreEqual("Hi there1!", msg.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser1FullName, msg.RegistratorDisplayName);
            Assert.AreEqual(FixtureConst.DocType1Name, msg.DocTypeName);
            Assert.AreEqual(ResourceString.UnderConsDocStatus, msg.DocStatusText);
            Assert.AreEqual(
                new DateTime(2007, 06, 07, 16, 13, 00),
                msg.NotificationTime
                );
            Assert.AreEqual(FixtureConst.DocCircUser1FullName, msg.LastActionUserDisplayName);
            Assert.AreEqual(
                DBConstKeeper.Current.NotificationType.NextDecision,
                msg.NotificationType
                );
            Assert.IsTrue(msg.InstantMessaging);
            Assert.AreEqual(FixtureConst.DocCircUser2ImAddress, msg.ImAddress);
            Assert.AreEqual(FixtureConst.DocCard1RoutePointNum, msg.RoutePointNum);
        }


    }
}
