//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Biz;

namespace Evpo.DocCircUnitTest
{

    [TestFixture]
    public class SecurityDataTest
    {
        protected struct FixtureConst
        {
            public const int DocCircUser1 = 0;
            public const int DocCircUser2 = 1;
            public const int DocCircUser3 = 2;

            public const long DocCard1 = 0;
            public const long DocCard2 = 1;
            public const long DocCard3 = 2;

            public const int DocType1 = 0;
            public const int DocType2 = 1;

            public const int DocCardFolder1 = 0;
            public const int DocCardFolder2 = 1;
        }

        protected class FixtureData
        {
            public static UserInfo DocCircUser1UserInfo;
            public static UserInfo DocCircUser2UserInfo;
            public static UserInfo DocCircUser3UserInfo;

            public static SecurityItem DocType1User1SI, DocType1User2SI;
            public static SecurityItem Folder1User1SI, Folder1User2SI;
        }

        protected class MockUserInfo : UserInfo
        {
            public MockUserInfo(int userId)
            {
                _userProperties = new UserProperties();
                _userProperties.UserId = userId;
            }

        }

        protected class MockSecurityAccess : SecurityAccess
        {
            public MockSecurityAccess(int userId)
            {
                _userId = userId;
            }

            public override SecurityItem GetDocTypeDocumentsSecurity(int docTypeId)
            {
                SecurityItem retVal = null;
                if (_userId == FixtureConst.DocCircUser1 && docTypeId == FixtureConst.DocType1)
                {
                    retVal = FixtureData.DocType1User1SI;
                }
                else
                {
                    throw new ApplicationException("Unknown id for this unit test!");
                }

                return retVal;
            }

            public override SecurityItem GetFolderSecurity(int folderId)
            {
                SecurityItem retVal = null;
                if (_userId == FixtureConst.DocCircUser1 && folderId == FixtureConst.DocCardFolder1)
                {
                    retVal = FixtureData.Folder1User1SI;
                }
                else
                {
                    throw new ApplicationException("Unknown id for this unit test!");
                }

                return retVal;
            }


        }

        protected class MockDataConnection : DataConnection
        {
            public MockDataConnection()
            {
                _con = null;
            }
        }

        protected class TestSecurityData : SecurityData
        {
            public TestSecurityData(UserInfo userInfo, DataConnection con):base(userInfo, con)
            {
            }

            protected override SecurityAccess GetSecurityAccess(int userId, DBConnection con)
            {
                return new MockSecurityAccess(userId);
            }
        }

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            FixtureData.DocCircUser1UserInfo = new MockUserInfo(FixtureConst.DocCircUser1);
        }

        [Test]
        public void GetDocumentCardSecurity()
        {
            /*
             *      C   R   U   D   B   RO  UO  DO
             * 1
             * 2    *                   *   *                                             
             * 3    *                   *   *   *                        
             * 4        *               *
             * 5    *   *               *   *
             * 6        *
             * 7        *   *
             * 8        *   *   *
             * 9    *   *   *   *       *   *   *
             * */
            
            // create permission test

            /* case 1
             * DocCircUser1 has 1 permission case in folder, and 9 permission case in DocType
             * 
             * */
            FixtureData.Folder1User1SI = new SecurityItem(false, false, false, false, false, false, false, false); //1
            FixtureData.DocType1User1SI = new SecurityItem(true, true, true, true, false, true, true, true); //9

            DocCardItem item = new DocCardItem();
            item.Id = FixtureConst.DocCard1;
            item.DocType = FixtureConst.DocType1;
            item.IsNew = false;
            item.Owner = FixtureConst.DocCircUser2;

            TestSecurityData sd = new TestSecurityData(FixtureData.DocCircUser1UserInfo, new MockDataConnection());
            SecurityItem si = sd.GetDocumentCardSecurity(item); //tested method

            Assert.IsNotNull(si);
            Assert.IsFalse(si.Create);
            Assert.IsFalse(si.Read);
            Assert.IsFalse(si.Update);
            Assert.IsFalse(si.Delete);
            Assert.IsFalse(si.ReadOwn);
            Assert.IsFalse(si.UpdateOwn);
            Assert.IsFalse(si.DeleteOwn);

            /* case 2
             * Folder: 2
             * DocType: 9
             * DocCircUser1 doesn't have permissions because Owner is DocCircUser2.
             * 
             * */
            FixtureData.Folder1User1SI = new SecurityItem(true, false, false, false, false, true, true, false); //2

            si = sd.GetDocumentCardSecurity(item); //tested method

            Assert.IsFalse(si.Read);
            Assert.IsFalse(si.Update);
            Assert.IsFalse(si.Delete);

            /* case 3
             * Folder: 9
             * DocType: 8
             * 
             * DocCard1 is new
             * DocCircUser1 can't create documents of this type, but can create in the folder
             * */

            item.IsNew = true;
            FixtureData.Folder1User1SI = FixtureData.DocType1User1SI;
            FixtureData.DocType1User1SI = new SecurityItem(false, true, true, true, false, false, false, false); //8

            si = sd.GetDocumentCardSecurity(item); //tested method

            Assert.IsTrue(si.Read);
            Assert.IsFalse(si.Update);
            Assert.IsTrue(si.Delete);

            /* case 4
             * Folder: 9
             * DocType: 3
             * 
             * DocCircUser1 has permissions because he is the owner of the document card
             * */
            item.IsNew = false;
            item.Owner = FixtureConst.DocCircUser1;

            FixtureData.DocType1User1SI = new SecurityItem(true, false, false, false, false, true, true, true); // 3

            si = sd.GetDocumentCardSecurity(item); //tested method

            Assert.IsTrue(si.Read);
            Assert.IsTrue(si.Update);
            Assert.IsTrue(si.Delete);

            /* case 4
             * Folder: 9
             * DocType: 3
             * 
             * DocCircUser1 doesn't have permissions because he is not the owner of the document card
             * */

            item.Owner = FixtureConst.DocCircUser2;
            
            si = sd.GetDocumentCardSecurity(item); //tested method

            Assert.IsFalse(si.Read);
            Assert.IsFalse(si.Update);
            Assert.IsFalse(si.Delete);

            /* case 5
             * Folder: 9
             * DocType: 9
             * 
             * DocCircUser1 have all permissions
             * */
            FixtureData.DocType1User1SI = FixtureData.Folder1User1SI;
            si = sd.GetDocumentCardSecurity(item); //tested method

            Assert.IsTrue(si.Read);
            Assert.IsTrue(si.Update);
            Assert.IsTrue(si.Delete);


            /* case 6
             * Folder: 9
             * DocType: not set
             * 
             * New document
             * */
            item.IsNew = true;
            item.DocType = null;
            item.Owner = FixtureConst.DocCircUser1;

            si = sd.GetDocumentCardSecurity(item); //tested method

            Assert.IsTrue(si.Read);
            Assert.IsTrue(si.Update);
            Assert.IsTrue(si.Delete);
        }

    }
}
