//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.IO;
using System.Resources;
using NUnit.Framework;
using Evpo.DocCirc.Messaging;
using Evpo.DocCirc.Common;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class EmailHtmlBodyComposerTest
    {
        public struct FixtureFile
        {
            public const string ApplicationFolder = "Messaging";
            public const string HtmlBodyTemplate = "EmailHtmlBodyTemplate.htm";
        }

        protected struct FixtureConst
        {
            public const string ServerName = "server-tst";
            public const string ApplicationUrl = "http://server.TEST.ORG/EvpoDc";
            public const int DocCircUser1 = 10;
            public const int DocCircUser2 = 11;
            public const int ServiceUserId = 12;
            public const string DocCircUser2Email = "doccircuser2@test.org";
            public const string ServiceUserEmail = "doccircservice@test.org";
            public const string ServiceUserFullName = "DocCirc service";
            public const string DocCircUser1FullName = "DocCirc user1";
            public const long DocCard1 = 21, DocCard2 = 22;
            public const int DocType1 = 23;
            public const string DocType1Name = "Doc type 1";
            public const long UserNotification1 = 31;
        }

        protected struct ResourceString
        {
            public const string InternalNumField = "Internal number:";
            public const string DocTypeNameField = "Document type:";
            public const string DocCardSubjectField = "Document card subject:";
            public const string DocCardStatusField = "Document card status:";
            public const string RegistratorFullNameField = "Registrator:";
            public const string OpenDocCardReference = "Open document card";
            public const string NotificationTimeField = "Notification time:";
            public const string LastUserWrote = "User {0} wrote:";
            public const string DocCardInformation = "Document card";
        }   


        protected struct DocCard1Property
        {
            public const string Subject = "doc card 1 subject";
            public const string InternalNum = "doc card 1 internal num";
            public const long Id = FixtureConst.DocCard1;
            public const int RegisteringUser = FixtureConst.DocCircUser1;
        }

        protected class MockResourceManager : ResourceManager
        {
            public override string GetString(string key)
            {
                string retVal = "";
                if (key == EmailHtmlBodyComposer.ResKeys.DocCardStatusField)
                {
                    retVal = ResourceString.DocCardStatusField;
                }
                else if (key == EmailHtmlBodyComposer.ResKeys.DocCardSubjectField)
                {
                    retVal = ResourceString.DocCardSubjectField;
                }
                else if (key == EmailHtmlBodyComposer.ResKeys.DocTypeNameField)
                {
                    retVal = ResourceString.DocTypeNameField;
                }
                else if (key == EmailHtmlBodyComposer.ResKeys.InternalNumField)
                {
                    retVal = ResourceString.InternalNumField;
                }
                else if (key == EmailHtmlBodyComposer.ResKeys.LastUserWrote)
                {
                    retVal = ResourceString.LastUserWrote;
                }
                else if (key == EmailHtmlBodyComposer.ResKeys.NotificationTimeField)
                {
                    retVal = ResourceString.NotificationTimeField;
                }
                else if (key == EmailHtmlBodyComposer.ResKeys.OpenDocCardReference)
                {
                    retVal = ResourceString.OpenDocCardReference;
                }
                else if (key == EmailHtmlBodyComposer.ResKeys.RegistratorFullNameField)
                {
                    retVal = ResourceString.RegistratorFullNameField;
                }
                else if (key == EmailHtmlBodyComposer.ResKeys.DocCardInformation)
                {
                    retVal = ResourceString.DocCardInformation;
                }
                else
                {
                    throw new ApplicationException(
                        String.Format("Unknown resource for this unit test: {0}!", key)
                        );
                }

                return retVal;
            }
        }

        protected class TestEmailHtmlBodyComposer : EmailHtmlBodyComposer
        {
            protected override ResourceManager GetResourceManager(string controlName)
            {
                return new MockResourceManager();
            }

            public TestEmailHtmlBodyComposer(SettingKeeper sk) : base(sk) { }
        }

        protected NameValueCollection _replaceResourceValues;
        protected SettingKeeper _sk;
        protected string _template;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            _sk = new SettingKeeper();
            _sk.ApplicationFolder = FixtureFile.ApplicationFolder;

            DBConstKeeper.Current = new DBConstKeeper();
            DBConstKeeper.Current.NotificationType = new NotificationTypeDirectory(1, 2, 3, 4, 5, 6, 7, 8, 9);

            //ResourceHelper.StrForUI = new MockStringResHelperInstance();
            _replaceResourceValues = new NameValueCollection();

            _replaceResourceValues.Add("^reskey:NotificationTimeField^", ResourceString.NotificationTimeField);
            _replaceResourceValues.Add("^reskey:InternalNumField^", ResourceString.InternalNumField);
            _replaceResourceValues.Add("^reskey:DocTypeNameField^", ResourceString.DocTypeNameField);
            _replaceResourceValues.Add("^reskey:DocCardSubjectField^", ResourceString.DocCardSubjectField);
            _replaceResourceValues.Add("^reskey:DocCardStatusField^", ResourceString.DocCardStatusField);
            _replaceResourceValues.Add("^reskey:RegistratorFullNameField^", ResourceString.RegistratorFullNameField);
            _replaceResourceValues.Add("^reskey:OpenDocCardReference^", ResourceString.OpenDocCardReference);

            //TextReader reader = File.OpenText(
            //    Path.Combine(FixtureFile.ApplicationFolder, FixtureFile.HtmlBodyTemplate)
            //    );
            //_template = reader.ReadToEnd();
            //reader.Close();
            _template = MessagingResources.EmailHtmlBodyTemplate;
        }

        [Test]
        public void CreateHtmlBody()
        {
            NotificationMessage msg = new NotificationMessage();
            msg.UserNotificationId = FixtureConst.UserNotification1;
            msg.Recipients = new string[] { FixtureConst.DocCircUser2Email };
            msg.NotificationType = DBConstKeeper.Current.NotificationType.ReturnToDraftDueToDecision;
            msg.NotificationTypeText = "The document card was returned to draft due to decision";
            msg.DocCardSubject = "doc card 1";
            msg.InternalNum = "TTF000001";
            msg.DocCardUrl = String.Format(
                "http://server-tst.test.org/DC/DocumentCard.aspx?id={0}",
                FixtureConst.DocCard1
                );
            msg.Text = "text1";
            msg.UserNote = "wrong document format";
            msg.LastActionUserDisplayName = "DocCirc User1";
            msg.RegistratorDisplayName = "DocCirc Admin";
            msg.DocTypeName = "Doc type first";
            msg.DocStatusText = "Under consideration";
            msg.NotificationTime = new DateTime(2007, 06, 09, 10, 34, 00);

            NameValueCollection replaceValues = new NameValueCollection();
            replaceValues.Add(
                "^NotificationTypeText^", 
                "The document card was returned to draft due to decision"
                );
            replaceValues.Add("^NotificationTime^", "09.06.2007 10:34:00");
            replaceValues.Add("^InternalNum^", "TTF000001");
            replaceValues.Add("^DocTypeName^", "Doc type first");
            replaceValues.Add("^DocCardSubject^", "doc card 1");
            replaceValues.Add("^DocCardStatus^", "Under consideration");
            replaceValues.Add("^RegistratorFullName^", "DocCirc Admin");

            replaceValues.Add("^LastUserWrote^", String.Format(ResourceString.LastUserWrote, "DocCirc User1"));
            replaceValues.Add("^LastUserNote^", "wrong document format");
            replaceValues.Add("^DocCardUrl^", msg.DocCardUrl);
            replaceValues.Add("^reskey:DocCardInformation^", ResourceString.DocCardInformation);
            replaceValues.Add("^control:LastUserDisplay^", "inline");
            string masterText = ReplaceValues(_template, _replaceResourceValues);
            masterText = ReplaceValues(masterText, replaceValues);
            Console.Write(masterText);
            TestEmailHtmlBodyComposer composer = new TestEmailHtmlBodyComposer(_sk);
            
            string resultText = composer.CreateHtmlBody(msg); // tested method

            Assert.IsNotNull(resultText);
            StringReader masterTextReader = new StringReader(masterText);
            StringReader resultTextReader = new StringReader(resultText);

            while (masterTextReader.Peek() >= 0)
            {
                Assert.GreaterOrEqual(resultTextReader.Peek(), 0);
                Assert.AreEqual(masterTextReader.ReadLine(), resultTextReader.ReadLine());
            }
        }

        protected string ReplaceValues(string template, NameValueCollection replaceValues)
        {
            foreach (string key in replaceValues.AllKeys)
            {
                template = template.Replace(key, replaceValues[key]);
            }
            return template;
        }
    }
}
