//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using MySql.Data.MySqlClient;
using NUnit.Framework;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class InstitutionAccessTest
    {
        public struct FixtureFile
        {
            public const string Folder = "MySql";
            public const string CreatingInstitutions = "InstitutionAccessTest.file1.sql";
            public const string AddInstitutionForDeletion = "InstitutionAccessTest.file2.sql";
        }

        protected class Tbl : DataTbl
        {
            public DataColumn InstitutionId = new DataColumn("InstitutionId", MySqlDbType.Int32, true, false);
            public DataColumn DcUser = new DataColumn("DcUser", MySqlDbType.Int32);
            public DataColumn Name = new DataColumn("Name", MySqlDbType.String);
            public DataColumn Phone1 = new DataColumn("Phone1", MySqlDbType.String);
            public DataColumn Phone2 = new DataColumn("Phone2", MySqlDbType.String);
            public DataColumn Address1 = new DataColumn("Address1", MySqlDbType.String);
            public DataColumn Address2 = new DataColumn("Address2", MySqlDbType.String);
            public DataColumn Email1 = new DataColumn("Email1", MySqlDbType.String);
            public DataColumn Email2 = new DataColumn("Email2", MySqlDbType.String);
            public DataColumn WebSite = new DataColumn("WebSite", MySqlDbType.String);
            public DataColumn Note = new DataColumn("Note", MySqlDbType.String);

            public Tbl()
            {
                Items = new DataColumn[]
                {
                    InstitutionId,
                    DcUser,
                    Name,
                    Phone1,
                    Phone2,
                    Address1,
                    Address2,
                    Email1,
                    Email2,
                    WebSite,
                    Note
                };
            }
        }

        protected Tbl _tbl = new Tbl();

        public class FixtureData
        {
            public static int DocCircUser1;
            public static int Institution1Id, Institution2Id;
        }

        private DBConnection _con;
        private MySqlTransaction _tran;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            _con = new DBConnection(
                "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
                );
            _con.Open();
            _tran = _con.MySqlConnection.BeginTransaction();

            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();
            TextReader txtReader = File.OpenText(Path.Combine(FixtureFile.Folder, FixtureFile.CreatingInstitutions));
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();
            cmd.ExecuteNonQuery();

            cmd.CommandText = "select cast(@docCircUser1 as unsigned) as DocCircUser1," +
                    "cast(@institution1 as unsigned) as Institution1, " +
                    "cast(@institution2 as unsigned) as Institution2;";
            MySqlDataReader reader = cmd.ExecuteReader();
            if(reader.Read())
            {
                FixtureData.DocCircUser1 = reader.GetInt32("DocCircUser1");
                FixtureData.Institution1Id = reader.GetInt32("Institution1");
                FixtureData.Institution2Id = reader.GetInt32("Institution2");
            }
            else
            {
                throw new ApplicationException("!");
            }

            reader.Close();

        }

        [Test]
        public void LoadInstitutionItem()
        {
            InstitutionAccess ia = new InstitutionAccess(FixtureData.DocCircUser1, _con);
            InstitutionItem item = ia.LoadInstitutionItem(FixtureData.Institution1Id); // tested method

            Assert.IsNotNull(item);
            Assert.AreEqual(FixtureData.Institution1Id, item.InstitutionId);
            Assert.AreEqual(FixtureData.DocCircUser1, item.Owner);
            Assert.AreEqual("Institution1", item.Name);
            Assert.AreEqual("1111-1", item.Phone1);
            Assert.AreEqual("1111-2", item.Phone2);
            Assert.AreEqual("FirstAddressOfInst1", item.Address1);
            Assert.AreEqual("SecondAddressOfInst1", item.Address2);
            Assert.AreEqual("inst11@test.org", item.Email1);
            Assert.AreEqual("inst12@test.org", item.Email2);
            Assert.AreEqual("inst1.test.org", item.WebSite);
            Assert.AreEqual("the best institution", item.Note);

            item = ia.LoadInstitutionItem(FixtureData.Institution2Id); // tested method

            Assert.AreEqual(FixtureData.Institution2Id, item.InstitutionId);
            Assert.AreEqual(FixtureData.DocCircUser1, item.Owner);
            Assert.AreEqual("Institution2", item.Name);
            Assert.AreEqual("2222-1", item.Phone1);
            Assert.AreEqual("2222-2", item.Phone2);
            Assert.AreEqual("FirstAddressOfInst2", item.Address1);
            Assert.AreEqual("SecondAddressOfInst2", item.Address2);
            Assert.AreEqual("inst21@test.org", item.Email1);
            Assert.AreEqual("inst22@test.org", item.Email2);
            Assert.AreEqual("inst2.test.org", item.WebSite);
            Assert.AreEqual("a better institution", item.Note);
        }

        [Test]
        public void GetAllInstitutionTokens()
        {
            InstitutionAccess ia = new InstitutionAccess(FixtureData.DocCircUser1, _con);
            InstitutionToken[] tokens = ia.GetAllInstitutionTokens();
            
            Assert.AreEqual(2, tokens.Length);

            bool[] been = new bool[] { false, false };
            foreach (InstitutionToken token in tokens)
            {
                if (token.Id == FixtureData.Institution1Id && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual("Institution1", token.Name);
                }
                else if (token.Id == FixtureData.Institution2Id && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual("Institution2", token.Name);
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }
            }
        }

        [Test]
        public void SaveInstitution()
        {
            InstitutionItem item = new InstitutionItem(
                FixtureData.DocCircUser1,
                "Institution3",
                "3333-1",
                "3333-2",
                "addr1",
                "addr2",
                "email1@test.org",
                "email2@test.org",
                "www.test.org",
                "new institution"
                );
            
            InstitutionAccess ia = new InstitutionAccess(FixtureData.DocCircUser1, _con);
            ia.SaveInstitution(item); // tested method

            Assert.IsNotNull(item.InstitutionId);

            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();

            StringBuilder sb = new StringBuilder();
            sb.Append("select ");
            bool first = true;
            foreach (DataColumn col in _tbl.Items)
            {
                if (first)
                {
                    first = false;
                }
                else
                {
                    sb.Append(", ");
                }
                sb.Append(col.Name);
            }

            sb.AppendFormat(" from Institution where {0}=?{0};", _tbl.InstitutionId.Name);

            cmd.CommandText = sb.ToString();
            _tbl.InstitutionId.SetParameter(cmd, item.InstitutionId);

            MySqlDataReader reader = cmd.ExecuteReader();

            try
            {
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(item.Name, reader.GetString(_tbl.Name.Name), "Name");
                Assert.AreEqual(item.Owner, reader.GetInt32(_tbl.DcUser.Name), "DcUser");
                Assert.AreEqual(item.Phone1, reader.GetString(_tbl.Phone1.Name), "Phone1");
                Assert.AreEqual(item.Phone2, reader.GetString(_tbl.Phone2.Name), "Phone2");
                Assert.AreEqual(item.Address1, reader.GetString(_tbl.Address1.Name), "Address1");
                Assert.AreEqual(item.Address2, reader.GetString(_tbl.Address2.Name), "Address2");
                Assert.AreEqual(item.Email1, reader.GetString(_tbl.Email1.Name), "Email1");
                Assert.AreEqual(item.Email2, reader.GetString(_tbl.Email2.Name), "Email2");
                Assert.AreEqual(item.WebSite, reader.GetString(_tbl.WebSite.Name), "WebSite");
                Assert.AreEqual(item.Note, reader.GetString(_tbl.Note.Name), "Note");
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }
        }

        [Test]
        public void DeleteInstitution()
        {
            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();
            TextReader txtReader = File.OpenText(
                Path.Combine(FixtureFile.Folder, FixtureFile.AddInstitutionForDeletion)
                );
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();

            cmd.ExecuteNonQuery();
            int newInstitutionId = (int)cmd.LastInsertedId;

            InstitutionAccess ia = new InstitutionAccess(FixtureData.DocCircUser1, _con);

            ia.DeleteInstitution(newInstitutionId); //tested method

            cmd.CommandText =
                String.Format("select {0} from Institution where {0}=?{0};", _tbl.InstitutionId.Name);
            _tbl.InstitutionId.SetParameter(cmd, newInstitutionId);
            MySqlDataReader reader = cmd.ExecuteReader();

            try
            {
                Assert.IsFalse(reader.Read());
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }

                cmd.CommandText =
                    String.Format("delete from Institution where {0}=?{0};", _tbl.InstitutionId.Name);
                cmd.ExecuteNonQuery();
            }





        }

        [TestFixtureTearDown]
        public void TFTearDown()
        {
            _tran.Rollback();
            _con.Close();
        }
    }
}
