//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using MySql.Data.MySqlClient;
using NUnit.Framework;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;

namespace Evpo.DocCircUnitTest
{

    [TestFixture]
    public class RoleAccessTest
    {
        protected struct FixtureConst
        {
            public const string Folder = "MySql";
            public const string CreateRolesAndAddUsersSql = "RoleAccessTest.file1.sql";
            public const string CreateRoleToDeleteSql = "RoleAccessTest.file2.sql";
            public const string CreateRoleToDeleteClearUpSql = "RoleAccessTest.file3.sql";
        }

        protected class FixtureData
        {
            public static int AdminRole, UserRole, TestUserRole, TestUserRole2;
            public static int DocCircUser1, DocCircUser2, DocCircUser3;
        }

        DBConnection _dbCon;
        MySqlTransaction _tran;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            _dbCon = new DBConnection(
                "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
                );
            _dbCon.Open();

            _tran = _dbCon.MySqlConnection.BeginTransaction();

            MySqlCommand cmd = _dbCon.MySqlConnection.CreateCommand();

            TextReader txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.CreateRolesAndAddUsersSql)
                );
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();

            cmd.ExecuteNonQuery();
            cmd.CommandText =
                "select cast(@TestUsersRoleId as unsigned), " +
                "cast(@TestUsers2RoleId as unsigned), " +
                "cast(@adminRole as unsigned), " +
                "cast(@userRole as unsigned), " +
                "cast(@docCircUser1 as unsigned), " +
                "cast(@docCircUser2 as unsigned), " +
                "cast(@docCircUser3 as unsigned); ";
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            FixtureData.TestUserRole = reader.GetInt32(0);
            FixtureData.TestUserRole2 = reader.GetInt32(1);
            FixtureData.AdminRole = reader.GetInt32(2);
            FixtureData.UserRole = reader.GetInt32(3);
            FixtureData.DocCircUser1 = reader.GetInt32(4);
            FixtureData.DocCircUser2 = reader.GetInt32(5);
            FixtureData.DocCircUser3 = reader.GetInt32(6);
            reader.Close();

            
        }

        [Test]
        public void GetUserRole()
        {
            RoleAccess ra = new RoleAccess(_dbCon);

            UserRole ur = ra.GetUserRole(FixtureData.TestUserRole);

            Assert.AreEqual(FixtureData.TestUserRole, ur.RoleId);
            Assert.AreEqual("TestUsers", ur.RoleName);

            ur = ra.GetUserRole(FixtureData.TestUserRole2);
            Assert.AreEqual(FixtureData.TestUserRole2, ur.RoleId);
            Assert.AreEqual("TestUsers2", ur.RoleName);
        }

        [Test]
        public void SaveUserRole()
        {
            // case 1
            // create a new role
            RoleAccess ra = new RoleAccess(_dbCon);
            UserRole role = new UserRole(null, "new role");

            ra.SaveUserRole(role); // tested method

            Assert.IsNotNull(role.RoleId);

            MySqlCommand cmd = _dbCon.MySqlConnection.CreateCommand();
            string keyCol = "DcRoleId";
            cmd.CommandText = String.Format(
                "select RoleName, count(*) as RowCount from DcRole where {0}=?{0} group by RoleName;",
                keyCol
                );
            cmd.Parameters.Add("?" + keyCol, MySqlDbType.Int32);
            cmd.Parameters["?" + keyCol].Value = role.RoleId;
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            int count = reader.GetInt32("RowCount");
            string roleName = reader.GetString("RoleName");
            bool violation = false;
            try
            {
                Assert.IsFalse(reader.Read());
                Assert.AreEqual(1, count);
                Assert.AreEqual("new role", roleName);
            }
            catch (Exception ex)
            {
                violation = true;
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
                if (violation)
                {
                    cmd.CommandText = String.Format(
                         "delete from RoleToUser where DcRole=?{0};" +
                         "delete from DcRole where {0}=?{0};",
                         keyCol
                         );
                    cmd.ExecuteNonQuery();
                }
            }

          

            // case 2
            // change role name
            role.RoleName = "new role changed";
            ra.SaveUserRole(role); // tested method

            cmd.CommandText = String.Format(
               "select RoleName, count(*) as RowCount from DcRole where {0}=?{0} group by RoleName;",
               keyCol
               );
            reader = cmd.ExecuteReader();
            Assert.IsTrue(reader.Read());
            count = reader.GetInt32("RowCount");
            roleName = reader.GetString("RoleName");
            try
            {
                Assert.IsFalse(reader.Read());
                Assert.AreEqual(1, count);
                Assert.AreEqual("new role changed", roleName);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
                cmd.CommandText = String.Format(
                    "delete from RoleToUser where DcRole=?{0};" + 
                    "delete from DcRole where {0}=?{0};", 
                    keyCol
                    );
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void DeleteUserRole()
        {
            MySqlCommand cmd = _dbCon.MySqlConnection.CreateCommand();

            TextReader txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.CreateRoleToDeleteSql)
                );
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();

            txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.CreateRoleToDeleteClearUpSql)
                );
            string clearUp = txtReader.ReadToEnd();
            txtReader.Close();

            cmd.ExecuteNonQuery();

            cmd.CommandText = "select cast(@TestUsers90RoleId as unsigned);";
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            int role90 = reader.GetInt32(0);
            reader.Close();

            RoleAccess ra = new RoleAccess(_dbCon);
            ra.DeleteUserRole(role90); // tested method
            string keyCol = "DcRoleId";
            cmd.CommandText = String.Format(
                "select count(*) from DcRole where {0}=?{0};",
                keyCol
                );
            cmd.Parameters.Add("?" + keyCol, MySqlDbType.Int32);
            cmd.Parameters["?" + keyCol].Value = role90;

            reader = cmd.ExecuteReader();
            reader.Read();
            int count = reader.GetInt32(0);
            reader.Close();

            try
            {
                Assert.AreEqual(0, count);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.CommandText = clearUp;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void GetAllUserRoles()
        {
            RoleAccess ra = new RoleAccess(_dbCon);
            UserRole[] roles = ra.GetAllUserRoles(); // test method

            Assert.AreEqual(4, roles.Length);

            bool[] been = new bool[] { false, false, false, false };

            foreach (UserRole role in roles)
            {
                if (role.RoleId == FixtureData.AdminRole && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual("Administrators", role.RoleName);
                }
                else if (role.RoleId == FixtureData.UserRole && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual("Users", role.RoleName);
                }
                else if (role.RoleId == FixtureData.TestUserRole && !been[2])
                {
                    been[2] = true;
                    Assert.AreEqual("TestUsers", role.RoleName);
                }
                else if (role.RoleId == FixtureData.TestUserRole2 && !been[3])
                {
                    been[3] = true;
                    Assert.AreEqual("TestUsers2", role.RoleName);
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }
            }


        }

        [Test]
        public void GetUserRoleUsers()
        {
            RoleAccess ra = new RoleAccess(_dbCon);
            UserToken[] users = ra.GetUserRoleUsers(FixtureData.TestUserRole);
            Assert.AreEqual(2, users.Length);

            bool[] been = new bool[] { false, false };
            foreach (UserToken user in users)
            {
                if (user.UserId == FixtureData.DocCircUser1 && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual("DocCircUser1", user.Login);
                }
                else if (user.UserId == FixtureData.DocCircUser2 && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual("DocCircUser2", user.Login);
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }
            }

            users = ra.GetUserRoleUsers(FixtureData.TestUserRole2);
            Assert.AreEqual(1, users.Length);
            Assert.AreEqual(FixtureData.DocCircUser3, users[0].UserId);
            Assert.AreEqual("DocCircUser3", users[0].Login);

        }

        [Test]
        public void SetUserRoleUsers()
        {
            MySqlCommand cmd = _dbCon.MySqlConnection.CreateCommand();
            TextReader txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.CreateRoleToDeleteSql)
                );
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();

            txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.CreateRoleToDeleteClearUpSql)
                );
            string clearUp = txtReader.ReadToEnd();
            txtReader.Close();

            cmd.ExecuteNonQuery();

            cmd.CommandText = "select cast(@TestUsers90RoleId as unsigned);";
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            int role90 = reader.GetInt32(0);
            reader.Close();

            // case 1
            // clear users in the role
            UserToken[] users = new UserToken[0];
            RoleAccess ra = new RoleAccess(_dbCon);
            ra.SetUserRoleUsers(role90, users); // tested method

            string keyCol = "DcRoleId";
            cmd.CommandText = String.Format(
                "select count(*) from RoleToUser where DcRole=?{0};",
                keyCol
                );

            cmd.Parameters.Add("?" + keyCol, MySqlDbType.Int32);
            cmd.Parameters["?" + keyCol].Value = role90;

            reader = cmd.ExecuteReader();
            reader.Read();
            int count = reader.GetInt32(0);
            reader.Close();

            try
            {
                Assert.AreEqual(0, count);
                // case 2
                // add two users to the role
                users = new UserToken[]
                {
                    new UserToken(FixtureData.DocCircUser1, "DocCircUser1"),
                    new UserToken(FixtureData.DocCircUser3, "DocCircUser3")
                };
                
                ra.SetUserRoleUsers(role90, users); // tested method

                cmd.CommandText = String.Format("select RoleToUserId, DcUser from RoleToUser where DcRole=?{0};", keyCol);
                reader = cmd.ExecuteReader();
                List<int> userIds = new List<int>();
                int docCircUser1RoleToUserId = 0, docCircUser3RoleToUserId = 1;
                while (reader.Read())
                {
                    int userId = reader.GetInt32("DcUser");
                    userIds.Add(userId);
                    if (userId == FixtureData.DocCircUser1)
                    {
                        docCircUser1RoleToUserId = reader.GetInt32("RoleToUserId");
                    }
                    else if (userId == FixtureData.DocCircUser3)
                    {
                        docCircUser3RoleToUserId = reader.GetInt32("RoleToUserId");
                    }
                }
                reader.Close();

                Assert.AreEqual(2, userIds.Count);
                Assert.IsTrue(userIds.Contains(FixtureData.DocCircUser1));
                Assert.IsTrue(userIds.Contains(FixtureData.DocCircUser3));

                //case 3
                // add a new user

                users = new UserToken[]
                {
                    new UserToken(FixtureData.DocCircUser1, "DocCircUser1"),
                    new UserToken(FixtureData.DocCircUser3, "DocCircUser3"),
                    new UserToken(FixtureData.DocCircUser2, "DocCircUser2")
                };

                ra.SetUserRoleUsers(role90, users); // tested method

                cmd.CommandText = 
                    String.Format("select DcUser, RoleToUserId from RoleToUser where DcRole=?{0};", keyCol);
                reader = cmd.ExecuteReader();
                userIds = new List<int>();
                while (reader.Read())
                {
                    int userId = reader.GetInt32("DcUser");
                    userIds.Add(userId);
                    if (userId == FixtureData.DocCircUser1)
                    {
                        Assert.AreEqual(docCircUser1RoleToUserId, reader.GetInt32("RoleToUserId"));
                    }
                    else if (userId == FixtureData.DocCircUser3)
                    {
                        Assert.AreEqual(docCircUser3RoleToUserId, reader.GetInt32("RoleToUserId"));
                    }
                }
                reader.Close();

                Assert.AreEqual(3, userIds.Count);
                Assert.IsTrue(userIds.Contains(FixtureData.DocCircUser1));
                Assert.IsTrue(userIds.Contains(FixtureData.DocCircUser3));
                Assert.IsTrue(userIds.Contains(FixtureData.DocCircUser2));

               
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
                cmd.CommandText = clearUp;
                cmd.ExecuteNonQuery();
            }




        }

        [TestFixtureTearDown]
        public void TFTearDown()
        {
            _tran.Rollback();
            _dbCon.Close();
        }

    }
}
