//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation 
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.IO;
using MySql.Data;
using MySql.Data.MySqlClient;
using NUnit.Framework;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;



namespace Evpo.DocCircUnitTest
{

    [TestFixture]
    public class UserAccessObject
    {
        protected struct FixtureConst
        {
            public const string Folder = "MySql";
            public const string CreateUserSqlFile = "UserAccessTest.file1.sql";
            public const string CreateUserForDeletionSqlFile = "UserAccessTest.file2.sql";
            public const string DeleteUserForDeletionSqlFile = "UserAccessTest.file3.sql";
        }

        MySqlTransaction _tran;
        int _userRole, _adminRole;
        int _user1Id;
        int _user2Id;
        int _user3Id;
        int _docCircAdmin;
        int _testUsersRoleId;
        int _testUsers2RoleId;
        int _user3Deputy1, _user3Deputy2;
        int _commonDepartment;
        DBConnection _dbCon;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            _dbCon = new DBConnection(
                "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
                );
            _dbCon.Open();

            _tran = _dbCon.MySqlConnection.BeginTransaction();

            TextReader textReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.CreateUserSqlFile)
                );
            string cmdText = textReader.ReadToEnd();
            textReader.Close();

            MySqlCommand cmd = new MySqlCommand(cmdText,_dbCon.MySqlConnection);
            cmd.ExecuteNonQuery();
            cmd.CommandText =
                "select cast(@User1Id as unsigned), cast(@User2Id as unsigned), cast(@User3Id as unsigned)," +
                "cast(@TestUsersRoleId as unsigned), cast(@TestUsers2RoleId as unsigned), " +
                "cast(@user3Deputy1 as unsigned), cast(@user3Deputy2 as unsigned), " +
                "cast(@commonDept as unsigned), cast(@userRole as unsigned), cast(@adminRole as unsigned), " +
                "cast(@docCircAdmin as unsigned); ";
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            _user1Id = reader.GetInt32(0);
            _user2Id = reader.GetInt32(1);
            _user3Id = reader.GetInt32(2);
            _testUsersRoleId = reader.GetInt32(3);
            _testUsers2RoleId = reader.GetInt32(4);
            _user3Deputy1 = reader.GetInt32(5);
            _user3Deputy2 = reader.GetInt32(6);
            _commonDepartment = reader.GetInt32(7);
            _userRole = reader.GetInt32(8);
            _adminRole = reader.GetInt32(9);
            _docCircAdmin = reader.GetInt32(10);
            reader.Close();

            DBConstKeeper.Current = new DBConstKeeper();
            DBConstKeeper.Current.MainRole = new MainRoleDirectory(_adminRole, _userRole);
        }

        [Test]
        public void GetUserPropertiesTest()
        {
           
            UserAccess userAccess = new UserAccess(_dbCon);
            UserProperties userProps = userAccess.GetUserProperties(_user1Id); // tested method
            
            Assert.AreEqual(
                _user1Id,
                userProps.UserId
                );

            Assert.AreEqual(
                "TestUser1",
                userProps.Login
                );

            Assert.AreEqual(
                "Test Test User1",
                userProps.FullName
                );

            Assert.IsFalse(userProps.AutomaticPass);
            Assert.IsTrue(userProps.AllowDeputyNotification);

            Assert.AreEqual(_commonDepartment, userProps.Department);

            Assert.IsTrue(userProps.InstantMessaging);
            Assert.AreEqual("testuser1@im.test.org", userProps.ImAddress);

            userProps = userAccess.GetUserProperties(_user2Id); // tested method
            
            Assert.AreEqual(
                _user2Id,
                userProps.UserId
                );

            Assert.AreEqual(
                "TestUser2",
                userProps.Login
                );

            Assert.AreEqual(
                "Test Test User2",
                userProps.FullName
                );

            Assert.IsNull(userProps.LastDocType);

            Assert.IsTrue(userProps.AutomaticPass);
            Assert.IsTrue(userProps.AllowDeputyNotification);

            Assert.AreEqual(_commonDepartment, userProps.Department);
            
            Assert.IsFalse(userProps.InstantMessaging);
            Assert.IsEmpty(userProps.ImAddress);
            
        }

        [Test]
        public void GetUserEmailsTest()
        {
            UserAccess userAccess = new UserAccess(_dbCon);
            string[] emails = userAccess.GetUserEmails(_user1Id);

            Assert.AreEqual(
                2,
                emails.Length
                );

            Assert.Contains(
                "test11@test.ru",
                emails
                );

            Assert.Contains(
                "test12@test.ru",
                emails
                );

            emails = userAccess.GetUserEmails(_user2Id);

            Assert.AreEqual(
                3,
                emails.Length
                );

            Assert.Contains(
                "test21@test.ru",
                emails
                );

            Assert.Contains(
                "test22@test.ru",
                emails
                );
            
            Assert.Contains(
                "test23@test.ru",
                emails
                );

            emails = userAccess.GetUserEmails(_user3Id);
            Assert.AreEqual(
                0,
                emails.Length
                );
        }

        [Test]
        public void GetUserRolesTest()
        {
            UserAccess ua = new UserAccess(_dbCon);
            UserRole[] userRoles = ua.GetUserRoles(_user2Id);

            Assert.IsNotNull(userRoles);

            Assert.AreEqual(2, userRoles.Length);

            bool[] cameIn = new bool[] { false, false };
            foreach (UserRole role in userRoles)
            {
                if (role.RoleId == _testUsersRoleId && !cameIn[0])
                {
                    cameIn[0] = true;
                    Assert.AreEqual("TestUsers", role.RoleName);
                }
                else if (role.RoleId == _testUsers2RoleId && !cameIn[1])
                {
                    cameIn[1] = true;
                    Assert.AreEqual("TestUsers2", role.RoleName);
                }
                else
                {
                    Assert.Fail("Wrong or repeated role id!");
                }
            }

            ua = new UserAccess(_dbCon);
            userRoles = ua.GetUserRoles(_user3Id);
            
            Assert.IsNotNull(userRoles);
            Assert.AreEqual(1, userRoles.Length);
            Assert.AreEqual(_testUsersRoleId, userRoles[0].RoleId);
            Assert.AreEqual("TestUsers", userRoles[0].RoleName);
        }
        

        [Test]
        public void GetAllUsersTest()
        {
            UserAccess userAccess = new UserAccess(_dbCon);
            int[] users = userAccess.GetAllUsers();
            
            Assert.GreaterOrEqual(
                users.Length,
                3
                );

            Assert.Contains(
                _user1Id,
                users
                );

            Assert.Contains(
                _user2Id,
                users
                );

            Assert.Contains(
                _user3Id,
                users
                );
        }

        [Test]
        public void GetAllUserProperties()
        {
            UserAccess userAccess = new UserAccess(_dbCon);
            UserProperties[] props = userAccess.GetAllUserProperties();
            
            Assert.IsNotNull(props);
            Assert.GreaterOrEqual(props.Length, 3);
            bool[] haveBeenHere = new bool[3]{false, false, false};
            foreach (UserProperties prop in props)
            {
                if (prop.UserId == _user1Id && !haveBeenHere[0])
                {
                    haveBeenHere[0] = true;
                    Assert.AreEqual("TestUser1", prop.Login);
                    Assert.AreEqual("Test Test User1", prop.FullName);
                    Assert.IsFalse(prop.AutomaticPass);
                    Assert.IsTrue(prop.AllowDeputyNotification);
                    Assert.AreEqual(_commonDepartment, prop.Department);
                    Assert.IsTrue(prop.InstantMessaging);
                    Assert.AreEqual("testuser1@im.test.org", prop.ImAddress);
                    Assert.IsNull(prop.LastDocType);
                }
                else if (prop.UserId == _user2Id && !haveBeenHere[1])
                {
                    haveBeenHere[1] = true;
                    Assert.AreEqual("TestUser2", prop.Login);
                    Assert.AreEqual("Test Test User2", prop.FullName);
                    Assert.IsTrue(prop.AutomaticPass);
                    Assert.IsTrue(prop.AllowDeputyNotification);
                    Assert.AreEqual(_commonDepartment, prop.Department);
                    Assert.IsFalse(prop.InstantMessaging);
                    Assert.IsEmpty(prop.ImAddress);
                    Assert.IsNull(prop.LastDocType);
                }
                else if (prop.UserId == _user3Id && !haveBeenHere[2])
                {
                    haveBeenHere[2] = true;
                    Assert.AreEqual("TestUser3", prop.Login);
                    Assert.AreEqual("Test Test User3", prop.FullName);
                    Assert.IsFalse(prop.AutomaticPass);
                    Assert.IsTrue(prop.AllowDeputyNotification);
                    Assert.AreEqual(_commonDepartment, prop.Department);
                    Assert.IsTrue(prop.InstantMessaging);
                    Assert.AreEqual("testuser3@im.test.org", prop.ImAddress);
                    Assert.IsNull(prop.LastDocType);
                }
                else if (prop.UserId > _user1Id)
                {
                    Assert.Fail(String.Format("Unknown or repeated user with id = {0}!", prop.UserId));
                }
            }
        }

        [Test]
        public void AuthorizeUserTest()
        {
            UserAccess userAccess = new UserAccess(_dbCon);
            UserToken token = userAccess.AuthorizeUser("TestUser2", "2");

            Assert.IsNotNull(token);

            Assert.AreEqual(
                _user2Id,
                token.UserId
                );

            token = userAccess.AuthorizeUser("TestUser1", "wrong password");

            Assert.IsNull(token);

            token = userAccess.AuthorizeUser("wrong user", "");

            Assert.IsNull(token);
        }

        [Test]
        public void GetUserIdByLoginTest()
        {
            UserAccess userAccess = new UserAccess(_dbCon);

            Assert.AreEqual(
                _user2Id,
                userAccess.GetUserIdByLogin("TestUser2")
                );
        }
        
        [Test]
        public void GetUserIdByExternalIdentityTest()
        {
            UserAccess userAccess = new UserAccess(_dbCon);

            Assert.AreEqual(
                _user2Id,
                userAccess.GetUserIdByExternalIdentity(@"TEST\TestUser2")
                );

            Assert.AreEqual(
                _user3Id,
                userAccess.GetUserIdByExternalIdentity(@"TEST\TestUser3")
                );
        }


        [Test]
        public void GetUserDeputies()
        {
            UserAccess userAccess = new UserAccess(_dbCon);
            DeputyItem[] items = userAccess.GetUserDeputies(_user3Id);
            
            Assert.AreEqual(2, items.Length);

            bool[] been = new bool[] { false, false };
            foreach (DeputyItem item in items)
            {
                if (item.DeputyId == _user3Deputy1 && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual(_user1Id, item.Deputy);
                    Assert.AreEqual(_user3Id, item.User);
                    Assert.IsTrue(item.Notify);
                }
                else if (item.DeputyId == _user3Deputy2 && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual(_user2Id, item.Deputy);
                    Assert.AreEqual(_user3Id, item.User);
                    Assert.IsTrue(item.Notify);
                }
                else
                {
                    throw new ApplicationException("Unknown or repeated deputy!");
                }
            }

        }


        [Test]
        public void GetWhomUserIsDeputyFor()
        {
            UserAccess userAccess = new UserAccess(_dbCon);
            DeputyItem[] items = userAccess.GetWhomUserIsDeputyFor(_user1Id);

            Assert.AreEqual(1, items.Length);
            Assert.AreEqual(_user3Deputy1, items[0].DeputyId);
            Assert.AreEqual(_user3Id, items[0].User);
            Assert.AreEqual(_user1Id, items[0].Deputy);
            Assert.IsTrue(items[0].Notify);
        }

        [Test]
        public void SaveUserProperties()
        {
            UserProperties userProp = new UserProperties();
            userProp.Login = "newuser";
            userProp.FullName = "New User";
            userProp.AutomaticPass = false;
            userProp.AllowDeputyNotification = true;
            userProp.Department = _commonDepartment;
            userProp.InstantMessaging = true;
            userProp.ImAddress = "newuser@im.test.org";
            userProp.ChangePwdNextTime = false;
            userProp.ExternalAuthorization = true;
            userProp.ExternalIdentity = @"\\TEST.ORG\newuser";

            UserAccess ua = new UserAccess(_dbCon);
            ua.SaveUserProperties(userProp); // tested method

            Assert.IsNotNull(userProp.UserId);

            string selectInDcUser =
                "select Login, Password, FullName, AutomaticPass, " +
                "AllowDeputyNotification, Department, InstantMessaging, ImAddress, " +
                "ChangePwdNextTime, ExternalAuthorization, ExternalIdentity " +
                "from DcUser where DcUserId=?DcUser;";

            string selectInDcRole = "select DcRole from RoleToUser where DcUser=?DcUser;";


            MySqlCommand cmd = new MySqlCommand(
                selectInDcUser,
                _dbCon.MySqlConnection
                );
            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters["?DcUser"].Value = userProp.UserId;
            MySqlDataReader reader = cmd.ExecuteReader();
            
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(userProp.Login, reader.GetString("Login"));
                Assert.AreEqual(userProp.FullName, reader.GetString("FullName"));
                Assert.IsFalse(reader.GetBoolean("AutomaticPass"));
                Assert.IsTrue(reader.GetBoolean("AllowDeputyNotification"));
                Assert.AreEqual(userProp.Department, reader.GetInt32("Department"));
                Assert.IsTrue(reader.GetBoolean("InstantMessaging"));
                Assert.AreEqual("newuser@im.test.org", reader.GetString("ImAddress"));
                Assert.AreEqual(false, reader.GetBoolean("ChangePwdNextTime"));
                Assert.AreEqual(true, reader.GetBoolean("ExternalAuthorization"));
                Assert.AreEqual(@"\\TEST.ORG\newuser", reader.GetString("ExternalIdentity"));

                Assert.IsFalse(reader.Read());
                reader.Close();
                
                cmd.CommandText = selectInDcRole;
                reader = cmd.ExecuteReader();

                Assert.IsTrue(reader.Read());
                Assert.AreEqual(_userRole, reader.GetInt32("DcRole"));
                Assert.IsFalse(reader.Read());
                reader.Close();

                int userId = userProp.UserId.Value;
                userProp.AutomaticPass = true;
                userProp.AllowDeputyNotification = false;
                userProp.FullName = "New User1";
                userProp.Login = "newuser1";
                userProp.ImAddress = "newuser1@im.test.org";
                userProp.ChangePwdNextTime = true;
                userProp.ExternalAuthorization = true;
                userProp.ExternalIdentity = @"\\TEST.ORG\newuser1";

                
                // check saving of an existent user
                ua.SaveUserProperties(userProp); // tested method
                
                Assert.AreEqual(userId, userProp.UserId);
                cmd.CommandText = selectInDcUser;
                reader = cmd.ExecuteReader();
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(userProp.Login, reader.GetString("Login"));
                Assert.AreEqual(userProp.FullName, reader.GetString("FullName"));
                Assert.IsTrue(reader.GetBoolean("AutomaticPass"));
                Assert.IsFalse(reader.GetBoolean("AllowDeputyNotification"));
                Assert.AreEqual(userProp.Department, reader.GetInt32("Department"));
                Assert.IsTrue(reader.GetBoolean("InstantMessaging"));
                Assert.AreEqual("newuser1@im.test.org", reader.GetString("ImAddress"));
                Assert.AreEqual(true, reader.GetBoolean("ChangePwdNextTime"));
                Assert.AreEqual(true, reader.GetBoolean("ExternalAuthorization"));
                Assert.AreEqual(@"\\TEST.ORG\newuser1", reader.GetString("ExternalIdentity"));
                Assert.IsFalse(reader.Read());
                reader.Close();

                cmd.CommandText = selectInDcRole;
                reader = cmd.ExecuteReader();
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(_userRole, reader.GetInt32("DcRole"));
                Assert.IsFalse(reader.Read());
                reader.Close();
                try
                {
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }

                cmd.CommandText =
                    "delete from RoleToUser where DcUser=?DcUser;" +
                    "delete from DcUser where DcUserId=?DcUser;";
                cmd.ExecuteNonQuery();
            }

        }

        [Test]
        public void SaveUserDeputies()
        {
            UserAccess ua = new UserAccess(_dbCon);
            DeputyItem[] items = new DeputyItem[]
            {
                new DeputyItem(_user1Id, _user2Id, true),
                new DeputyItem(_user1Id, _user3Id, false)
            };
            ua.SaveUserDeputies(_user1Id, items); // tested method

            Assert.IsNotNull(items[0].DeputyId);
            Assert.IsNotNull(items[1].DeputyId);

            MySqlCommand cmd = new MySqlCommand(
                "select DeputyId, Deputy, Notify from Deputy where DcUser=?DcUser;",
                _dbCon.MySqlConnection
                );
            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters["?DcUser"].Value = _user1Id;

            MySqlDataReader reader = cmd.ExecuteReader();

            
                bool[] been = new bool[] { false, false };
                while (reader.Read())
                {
                    if (reader.GetInt32("DeputyId") == items[0].DeputyId && !been[0])
                    {
                        been[0] = true;
                        Assert.AreEqual(_user2Id, reader.GetInt32("Deputy"));
                        Assert.IsTrue(reader.GetBoolean("Notify"));
                    }
                    else if (reader.GetInt32("DeputyId") == items[1].DeputyId && !been[1])
                    {
                        been[1] = true;
                        Assert.AreEqual(_user3Id, reader.GetInt32("Deputy"));
                        Assert.IsFalse(reader.GetBoolean("Notify"));
                    }
                    else
                    {
                        throw new ApplicationException("Unknown or repeated deputy!");
                    }
                }
                reader.Close();
                Assert.IsTrue(been[0]);
                Assert.IsTrue(been[1]);
                items[0].Notify = false; // change first item
                items[1] = new DeputyItem(_user1Id, _docCircAdmin, true); // add new item
                ua.SaveUserDeputies(_user1Id, items); // tested method
            
                Assert.IsNotNull(items[0].DeputyId);
                Assert.IsNotNull(items[1].DeputyId);

                reader = cmd.ExecuteReader();
                been = new bool[] { false, false };
                while (reader.Read())
                {
                    if (reader.GetInt32("DeputyId") == items[0].DeputyId && !been[0])
                    {
                        been[0] = true;
                        Assert.AreEqual(_user2Id, reader.GetInt32("Deputy"));
                        Assert.IsFalse(reader.GetBoolean("Notify"));
                    }
                    else if (reader.GetInt32("DeputyId") == items[1].DeputyId && !been[1])
                    {
                        been[1] = true;
                        Assert.AreEqual(_docCircAdmin, reader.GetInt32("Deputy"));
                        Assert.IsTrue(reader.GetBoolean("Notify"));
                    }
                    else
                    {
                        throw new ApplicationException("Unknown or repeated deputy!");
                    }
                }
                reader.Close();
                Assert.IsTrue(been[0]);
                Assert.IsTrue(been[1]);
                try
                {
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }

                cmd.CommandText = "delete from Deputy where DcUser=?DcUser;";
                cmd.ExecuteNonQuery();
            }


        }

        [Test]
        public void DeleteUser()
        {
            TextReader txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.CreateUserForDeletionSqlFile)
                );

            MySqlCommand cmd = _dbCon.MySqlConnection.CreateCommand();
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();

            txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.DeleteUserForDeletionSqlFile)
                );
            string clearUpCmd = txtReader.ReadToEnd();
            txtReader.Close();


            cmd.ExecuteNonQuery();

            cmd.CommandText = "select cast(@User90Id as unsigned) as User90;";
              
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            int user90 = reader.GetInt32("User90");
            reader.Close();
                        
            UserAccess ua = new UserAccess(_dbCon);
            ua.DeleteUser(user90); // tested method

            cmd.CommandText = String.Format(
                "{0} DcUser where {1}=?{1} union " +
                "{0} UserEmail where DcUser=?{1} union " +
                "{0} RoleToUser where DcUser=?{1} union " +
                "{0} Deputy where DcUser=?{1};",
                "select count(*) as RowCount from",
                "DcUserId"
                );

            cmd.Parameters.Add("?DcUserId", MySqlDbType.Int32);
            cmd.Parameters["?DcUserId"].Value = user90;

            reader = cmd.ExecuteReader();
            bool violation = false;
            try
            {
                while (reader.Read())
                {
                    Assert.AreEqual(0, reader.GetInt32("RowCount"));
                }
            }
            catch (Exception ex)
            {
                violation = true;
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }

                if (violation)
                {
                    cmd.CommandText = clearUpCmd;
                    cmd.ExecuteNonQuery();
                }
            }
        }

        [Test]
        public void SaveUserEmails()
        {

            TextReader txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.CreateUserForDeletionSqlFile)
                );

            MySqlCommand cmd = _dbCon.MySqlConnection.CreateCommand();
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();

            txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.DeleteUserForDeletionSqlFile)
                );
            string clearUpCmd = txtReader.ReadToEnd();
            txtReader.Close();

            cmd.ExecuteNonQuery();

            cmd.CommandText = "select cast(@User90Id as unsigned) as User90;";

            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            int user90 = reader.GetInt32("User90");
            reader.Close();

            bool violation = false;
            try
            {
                
                // case 1 
                // delete all emails
                UserAccess ua = new UserAccess(_dbCon);
                string[] emails = new string[0];
                ua.SaveUserEmails(user90, emails); // tested method

                cmd.CommandText = "select count(*) from UserEmail where DcUser=?DcUserId;";
                cmd.Parameters.Add("?DcUserId", MySqlDbType.Int32);
                cmd.Parameters["?DcUserId"].Value = user90;
                reader = cmd.ExecuteReader();
                reader.Read();
                int count = reader.GetInt32(0);
                reader.Close();
                Assert.AreEqual(0, count);
                
                // case 2
                // add two emails
                emails = new string[]
                {
                    "user90e1@test.ru",
                    "user90e2@test.ru"
                };

                ua.SaveUserEmails(user90, emails); // tested method
                cmd.CommandText = "select UserEmailId, UserEmail from UserEmail where DcUser=?DcUserId;";
                reader = cmd.ExecuteReader();
                Dictionary<string, int> dict = new Dictionary<string, int>();
                while (reader.Read())
                {
                    dict.Add(reader.GetString("UserEmail"), reader.GetInt32("UserEmailId"));
                }
                reader.Close();

                Assert.AreEqual(2, dict.Count);
                Assert.IsTrue(dict.ContainsKey("user90e1@test.ru"));
                Assert.IsTrue(dict.ContainsKey("user90e2@test.ru"));

                //case 2
                // add two emails
                emails = new string[]
                {
                    "user90e1@test.ru",
                    "user90e2@test.ru",
                    "user90e3@test.ru",
                    "user90e4@test.ru"
                };
                
                ua.SaveUserEmails(user90, emails); // tested method
                
                Dictionary<string, int> dict2 = new Dictionary<string, int>();
                reader = cmd.ExecuteReader();
                while (reader.Read())
                {
                    dict2.Add(reader.GetString("UserEmail"), reader.GetInt32("UserEmailId"));
                }
                reader.Close();

                Assert.AreEqual(4, dict2.Count);
                Assert.IsTrue(dict2.ContainsKey("user90e1@test.ru"));
                Assert.IsTrue(dict2.ContainsKey("user90e2@test.ru"));
                Assert.IsTrue(dict2.ContainsKey("user90e3@test.ru"));
                Assert.IsTrue(dict2.ContainsKey("user90e4@test.ru"));

                Assert.AreEqual(dict["user90e1@test.ru"], dict2["user90e1@test.ru"]);
                Assert.AreEqual(dict["user90e2@test.ru"], dict2["user90e2@test.ru"]);
               
            }
            catch (Exception ex)
            {
                violation = true;
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }

                if (violation)
                {
                    cmd.CommandText = clearUpCmd;
                    cmd.ExecuteNonQuery();
                }
            }

        }

        [Test]
        public void SetUserPassword()
        {
            MySqlCommand cmd = _dbCon.MySqlConnection.CreateCommand();
            string clearUp = "update DcUser set Password=Password(?Password) where DcUserId=@User2Id;";
            cmd.Parameters.Add("?Password", MySqlDbType.String);

            UserAccess ua = new UserAccess(_dbCon);
            ua.SetUserPassword(_user2Id, "new password"); // tested method

            cmd.CommandText = "select count(*) from DcUser where DcUserId=@User2Id and Password=Password(?Password);";
            cmd.Parameters["?Password"].Value = "new password";

            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            int count = reader.GetInt32(0);
            reader.Close();

            try
            {
                Assert.AreEqual(1, count);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.CommandText = clearUp;
                cmd.Parameters["?Password"].Value = "2";
                cmd.ExecuteNonQuery();
            }



        }

        [TestFixtureTearDown]
        public void TFTearDown()
        {
            _tran.Rollback();
            _dbCon.Close();
        }


    }
}
