//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using NUnit.Framework;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class DocTypeCheckerTest
    {
        /// <summary>
        /// 
        /// </summary>
        public class MockDocTypeData : DocTypeData
        {
            public MockDocTypeData()
            {
            }

            public override int[] PrefixExists(string prefix)
            {
                int[] retVal = new int[1];
                switch (prefix)
                {
                    case DocType1.Prefix:
                        retVal[0] = DocType1.Id;
                        break;
                    case DocType2.Prefix:
                        retVal[0] = DocType2.Id;
                        break;
                    default:
                        retVal = new int[0];
                        break;
                }
                return retVal;
            }
        }

        public class MockStringResHelperInstance : StringResHelper
        {
            private NameValueCollection _resources = new NameValueCollection();
            public MockStringResHelperInstance()
            {
                _resources.Add(
                    "DocTypeCheckerTextInroduction",
                    "The document type cannot be saved due to the following:"
                    );
                _resources.Add(
                    "DocTypeCheckerPrefixIsNotUnique",
                    "Prefix {0} is not unique."
                    );
                _resources.Add(
                    "DocTypeCheckerMustBeginFromFirst",
                    "Route cannot begin from {0} route point number. The first route point number shoul be 1."
                    );
                _resources.Add(
                    "DocTypeCheckerRoutePointIsMissed",
                    "{0} route point is missed between route point numbers {1} and {2}."
                    );
                _resources.Add(
                    "DocTypeCheckerEmployeeMoreThanOnceOnRoutePoint",
                    "Route point {0} contains an employee more than once."
                    );

                _resources.Add(
                    "DocTypeCheckerRouteIsEmpty",
                    "Route is empty."
                    );
            }

            public override string this[string key]
            {
                get
                {
                    return _resources[key];
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public class TestDocTypeChecker : DocTypeChecker
        {
            public TestDocTypeChecker()
            {
                _serviceUserInfo = null;
                _con = null;
            }

            protected override DocTypeData GetDocTypeData(UserInfo userInfo, DataConnection con)
            {
                return new MockDocTypeData();
            }

        }


        public struct DocType1
        {
            public const string Prefix = "TTF";
            public const int Id = 10;
        }

        public struct DocType2
        {
            public const string Prefix = "TTS";
            public const int Id = 11;
        }

        public struct FixtureConst
        {
            public const int NewDocTypeId = 12;
            public const int RootDocTypeFolder = 1;
            public const int User1Id = 21;
            public const int User2Id = 22;
            public const int User3Id = 23;

            public const int DecisionRouteItemType = 31;
        }


        [TestFixtureSetUp]
        public void TFSetUp()
        {
            ResourceHelper.StrForEx = new MockStringResHelperInstance();
        }

        [Test]
        public void CheckDocTypeBeforeSave()
        {
            
            
            DocTypeItem docTypeToCheck = new DocTypeItem(
                FixtureConst.NewDocTypeId,
                FixtureConst.RootDocTypeFolder,
                "New DocType",
                "TTF1", // error: DocType1 has the same prefix
                "",
                1,
                FixtureConst.User1Id
                );
            docTypeToCheck.SetRoutePointsLoaded(true);


            TestDocTypeChecker dtch = new TestDocTypeChecker();
            //Route is empty
            CheckerAnswer answer = dtch.CheckDocTypeBeforeSave(docTypeToCheck); // tested method
            Assert.IsNotNull(answer);
            Assert.IsFalse(answer.Success);
            Assert.AreEqual(
                "The document type cannot be saved due to the following:<br/>" +
                "Route is empty.",
                answer.TextForUser
                );

            // new DocType with the same prefix as DocType1 has
            docTypeToCheck.Prefix = "TTF";
            docTypeToCheck.RoutePoints.Add(new RoutePoint(null, "[0]", 1));
            docTypeToCheck.RoutePoints[0].RouteItems.Add(
                new RouteItem(null, FixtureConst.User1Id, FixtureConst.DecisionRouteItemType)
                );

            
            
            answer = dtch.CheckDocTypeBeforeSave(docTypeToCheck); // tested method
            Assert.IsNotNull(answer);
            Assert.IsFalse(answer.Success);
            Assert.AreEqual(
                "The document type cannot be saved due to the following:<br/>" +
                "Prefix TTF is not unique.",
                answer.TextForUser
                );

            // change prefix for it being unique
            docTypeToCheck.Prefix = "TTF1"; 
            answer = dtch.CheckDocTypeBeforeSave(docTypeToCheck);
            Assert.IsNotNull(answer);
            Assert.IsTrue(answer.Success);
            Assert.IsEmpty(answer.TextForUser);

            //first route point OrderNum=2 and not unique prefix
            docTypeToCheck.Prefix = "TTF";
            docTypeToCheck.RoutePoints[0].OrderNum = 2;
            answer = dtch.CheckDocTypeBeforeSave(docTypeToCheck);
            Assert.IsNotNull(answer);
            Assert.IsFalse(answer.Success);
            Assert.AreEqual(
                "The document type cannot be saved due to the following:<br/>" +
                "Prefix TTF is not unique.<br/>" +
                "Route cannot begin from 2 route point number. The first route point number shoul be 1.",
                answer.TextForUser
                );

            // gap between route point numbers: 1 - 3 without 2
            docTypeToCheck.Prefix = "TTF1";
            docTypeToCheck.RoutePoints[0].OrderNum = 1;
            docTypeToCheck.RoutePoints.Add(
                new RoutePoint(null, "[0]", 3)
                );
            docTypeToCheck.RoutePoints[1].RouteItems.Add(
                new RouteItem(null, FixtureConst.User2Id, FixtureConst.DecisionRouteItemType)
                );
            answer = dtch.CheckDocTypeBeforeSave(docTypeToCheck);
            Assert.IsNotNull(answer);
            Assert.IsFalse(answer.Success);
            Assert.AreEqual(
                "The document type cannot be saved due to the following:<br/>" +
                "1 route point is missed between route point numbers 1 and 3.",
                answer.TextForUser
                );

            // employee more than once on 2 route point
            docTypeToCheck.RoutePoints[1].OrderNum = 2;
            docTypeToCheck.RoutePoints[1].RouteItems.Add(
                new RouteItem(null, FixtureConst.User2Id, FixtureConst.DecisionRouteItemType)
                );
            answer = dtch.CheckDocTypeBeforeSave(docTypeToCheck);
            Assert.IsNotNull(answer);
            Assert.IsFalse(answer.Success);
            Assert.AreEqual(
                "The document type cannot be saved due to the following:<br/>" +
                "Route point 2 contains an employee more than once.",
                answer.TextForUser
                );



        }



    }
}
